#!/bin/bash


# Experimental Annual Sunlight Exposure variant of lm83.sh version 1.1
# 
# Version NA (August 25, 2017)
# 
# Author: Alen Mahic, Ery Djunaedy (Energy Studies in Buildings Laboratory Universiy of Oregon; Integrated Design Lab University of Idaho)
# This work is licensed under the Creative Commons Attribution 3.0 Unported License. To view a copy of this license, visit http://creativecommons.org/licenses/by/3.0/.
# 
# Radiance 5.0+ is required. (NREL package)
# https://openstudio.nrel.gov/getting-started-developer/getting-started-radiance
# 
# Usage: ./idl_lm83.sh -i [file] -m N -rz+/-N -clean -rtrace
# 
#       -i [file]       The main .rad file for the model, which pulls its geometry elements from the ./objects/ directory
#                       and its windows from the ./objects/windows/ directory. Each window group is broken down into a
#                       .rad file of its own with the filename matching the modifier name  used for each defined polygon.
#                       These two must match. Required input.
# 
#       -m N            Specifies the number of subdivisions of the sky. The Reinhart MF: setting where "N" is 1, 2, 3,
#                       or 4. Optional input, the default value is 4.
# 
#       -rz+/-N         Specifies the orientation of the building in degrees. Make sure that there is no space between the
#                       "-rz" argument and the +/- "N" value. Optional input, the default value is 0.
# 
#       -clean          This switch will clean the directory and delete all directories and files that were generated in
#                       any previous runs. This includes the contents of the "results" folder.
# 
#       -rcontrib       This switch will enable the use of rcontrib for the direct sun analysis, otherwise it will use 
#                       rtrace with continuous skies generated by gendaylit (or gensky if gendaylit should fail) to
#                       determine the direct sun exposure for the solar disc analysis. This method is much more time
#                       consuming and takes ~10-15 times as long.
# 
#       -add            Inclusion of non-operated glazing/skylight openings.
# 
# Assumed folder structure:
# 
#       ./
#           The main directory contains this script file and the primary .rad file of the model.
# 
#       ./data/
#           Contains the point analysis files.
# 
#       ./materials/
#           Contains the "materials.rad" file with the relevant material definitions for the model. The "glazing" and
#           "wall_mat" material identifiers are used in the script and assigned to the glazings and walls respectively.
#           These two must be defined in the "materials.rad" file. This directory also includes the .xml BSDF input files.
# 
#       ./objects/
#           Contains the geometry .rad files referenced by the primary model file in the main directory.
# 
#       ./objetcs/windows/
#           Contains the window group definitions, each in an individual .rad file with the filename matching the modifier
#           name used for each defined polygon. These two must match. The modifier convention requires that each modifier
#           include one of the following suffixes:
#                                                   "_s" for a south-facing window group
#                                                   "_w" for a west-facing window group
#                                                   "_n" for a north-facing window group
#                                                   "_e" for an east-facing window group
# 
#                                  File example:    ./objects/windows/windows_s.rad
#                            Definition example:    windows_s polygon window_name
#                                                   0
#                                                   0
#                                                   12
#                                                       0 0 0
#                                                       2 0 0
#                                                       2 0 2
#                                                       0 0 2
# 
#       ./wea/
#           Contains the TMY3 weather files in .epw format.
# 
# Assumed input files:
# 
#       ./[filename].rad
#           Primary model file.
# 
#       ./data/*.pts
#           All desired analysis point files.
# 
#       ./materials/materials.rad
#           Materials file.
# 
#       ./materials/singleclear.xml
#           BSDF file for the open condition. The "singleclear" name is also assigned to the $bsdf_open variable in the script.
# 
#       ./materials/venetian_75.xml
#           BSDF file for the closed condition. The "venetian_75" name is also assigned to the $bsdf_closed variable in the script.
# 
#       ./objects/*.rad
#           All relevant geometry files referenced by the primary model file.
# 
#       ./objects/windows/*.rad
#           All desired window groups seperated into .rad files, taking their filenames from the modifiers used in each one.
# 
#       ./wea/*.epw
#           All desired weather files.
# 
# Results:
#       The outputs will be deposited in the ./results/ directory. The naming conventions for the operated annual illuminance
#       matrix, the operation schedule files, and the LM-83 metric calculations are as follows:
#           [pointfile]--[weatherfile]--m[N]--final.ill
#           [pointfile]--[weatherfile]--m[N]--final_blinds_sched.dat
#           LM83_metrics.dat
# 
# Non-operated glazing and/or skylight material assignments:
#       The following modifier names are to be assigned to geometries that are windows or skylights, but not part of any
#       operated window groups:
#           "nonoperated_glaz"    (Glazing)
#           "nonoperated_sky"     (Skylight)
#       Currently this script supports one material assignment for each, but can be modified to accomodate more variables
#       for both types.
# 
# Material options:
# Specify the identifier of the glazing material definition.. mat_glaz="glazing"
mat_glaz="glaz"
# Specify the identifier of the wall material definition.. mat_wall="wall_mat"
mat_wall="wall"
# Specify the name of the "open" condition BSDF file without the file extension.. bsdf_open="singleclear"
bsdf_open="singleclear"
# Specify the name of the "closed" condition BSDF file without the file extension.. bsdf_closed="venetian_75"
bsdf_closed="venetian75"
# Specify the name of the skylight material, assuming that "nonoperated_sky" has been assigned to some geometries in the ./objects folder.
mat_sky="glaz"
# If the -add argument is specified, the script will also look for the "nonoperated_glaz" modifier. These surfaces will be simulated separately of the operable window groups and then later added as the non-operated contribution to the total light in the space.

# Location of genklemsamp.pl Radiance perl script.
genklemsamp="C:\Program Files\Radiance\bin\genklemsamp.pl"



# Read script arguments..
args=$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/-\{1,\} /-/g;s/ -/#-/g")

# SOS..
help="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -x -e "-\{1,\}help" |tail -1 |sed -e "s/^-\{1,\}//")"
# Clean switch..
clean="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -x -e "-\{1,\}clean" |tail -1 |sed -e "s/^-\{1,\}//")"

# Blinds operation parameters..
sd_trigger="1000"
sd_trigger_per="2"
da_trigger="300"
sda_trigger="50"
ase_trigger_lux="1000"
ase_trigger_hours="250"
period_start="9"
period_end="18"
period_int=$(echo $period_end $period_start |awk '{printf("%.0f",$1-$2)}')

# Directories
windir="./objects/windows"
ptsdir="./data"
if [ ! "$(ls $ptsdir)" ];then ">> Error: The \"$ptsdir\" is empty.";exit 0;fi;
matdir="./materials"
if [ ! "$(ls $matdir)" ];then ">> Error: The \"$matdir\" is empty.";exit 0;fi;
weadir="./wea"
if [ ! "$(ls $wadir)" ];then ">> Error: The \"$weadir\" is empty.";exit 0;fi;

resdir="./results"
resdir_dat="./results/dat"
resdir_ill="./results/ill"
resdir_sd="./results/solardisc"
resdir_add="./results/add"
optdir="./options"
skydir="./skies"
tmpdir="./temp"
logdir="./log"

# Temp and material outputs..
dmx="./materials/inherit_dmx.rad"
vmx="./materials/inherit_vmx.rad"
solardisc="./materials/inherit_solardisc.rad"
tmp_solardisc="inherit_solardisc.rad"
tmp_additional="inherit_additional.rad"
tmp_sky="temp_sky.rad"
tmp_wea="temp_weather.wea"
tmp_dat="temp.dat"
tmp_ill="temp.ill"

scriptname=$(basename $0)

# Print help..
if [ "$help" ];then
    #help_man=$(head -108 $scriptname |tail -n+2 |sed -e "s/#/ /g")
    #echo -e $help_man
    head -98 $scriptname |tail -n+2 |sed -e "s/#/>/g" |more; echo;
    exit 0;
fi

# Clean the directory..
if [ "$clean" ];then
    rm -f -r $tmpdir $resdir $skydir $optdir $dmx $vmx $solardisc
    rmlist=$(ls ./ |grep -i -e ".oct$")
    for f in $rmlist; do rm -f -r $f;done;
    rmlist=$(ls $weadir |grep -v -i -e ".epw$")
    for f in $rmlist; do rm -f -r $weadir/$f;done;
    echo ">  Directory cleaned."
    exit 0;
fi

# Parse model and parameter arguments..
#model="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/\t-/g" |tr '\t' '\n' |grep -e "-\{1,\}i " |tail -1 |sed "s/-\{1,\}i //;s/^\.\{1,\}//;s/^\/\{1,\}//;s/^/\.\//;s/\/$//")"
model="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -e "-\{1,\}i " |tail -1 |sed "s/-\{1,\}i //;s/^\.\{1,\}//;s/^\/\{1,\}//;s/^/\.\//;s/\/$//")"
#if [ ! "$model" ];then echo -e ">> Error: Please specify the input model.\n    Help: ./$scriptname -help";exit 0;elif [ ! -f $model ];then echo ">> Error: The specified file does not exist.";exit 0;elif [ ! ".${model##*.}" == ".rad" ];then echo -e ">> Error: The specified file is not in .rad format.";exit 0;fi;
if [ ! "$model" ];then echo -e ">> Error: Please specify the input model.";exit 0;elif [ ! -f $model ];then echo ">> Error: The specified file does not exist.";exit 0;elif [ ! $(echo ".${model##*.}" |grep -e ".rad") ];then echo -e ">> Error: The specified file is not in .rad format.";exit 0;fi;
#granularity="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/\t-/g" |tr '\t' '\n' |grep -e "-\{1,\}m " |tail -1 |sed "s/-\{1,\}m //")"
granularity="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -e "-\{1,\}m " |tail -1 |sed "s/-\{1,\}m //" |grep -v -e "[^0-9]")"
if [ ! "$granularity" ];then granularity=4;	granularity=4;elif [ $granularity -gt 4 ] || [ $granularity -lt 1 ];then echo ">> Warning: The specified granularity is out of range, using default value. [4]";granularity=4;else echo ">> Sky sub-division granularity set to m $granularity.";fi;
#rotation="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/\t-/g" |tr '\t' '\n' |grep -e "-\{1,\}rz" |tail -1 |sed "s/-\{1,\}rz//")"
rotation="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -e "-\{1,\}rz" |tail -1 |sed "s/-\{1,\}rz//")"
if [ ! "$rotation" ];then rotation=0;rztag="";else rztag="--rz$rotation";fi;
#rtrace="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/\t-/g" |tr '\t' '\n' |grep -x -e "-\{1,\}rtrace" |tail -1 |sed -e "s/^-\{1,\}//")"
#rcontrib="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/\t-/g" |tr '\t' '\n' |grep -x -e "-\{1,\}rcontrib" |tail -1 |sed -e "s/^-\{1,\}//")"
rcontrib="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -e "-\{1,\}rcontrib" |tail -1 |sed "s/-\{1,\}//")"

# Model and materials..
modelname=$(basename $model .${model##*.})
materials="./materials/materials.rad"
if [ ! -f $materials ];then echo ">> Error: The \"$materials\" file does not exist.";exit 0;fi;
materials_check=$(cat $materials |sed $'s/\t*$//g;s/ *$//g' |grep -e "${mat_glaz}$")
if [ ! "$materials_check" ]; then echo ">> Error: The \"$mat_glaz\" material definition was not found in the \"$materials\" file.";exit 0;fi;
materials_check=$(cat $materials |sed $'s/\t*$//g;s/ *$//g' |grep -e "$mat_wall$")
if [ ! "$materials_check" ]; then echo ">> Error: The \"$mat_wall\" material definition was not found in the \"$materials\" file.";exit 0;fi;
add="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -e "-\{1,\}add" |tail -1 |sed "s/-\{1,\}//")"
if [ "$mat_sky" ];then
    #add="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/\t-/g" |tr '\t' '\n' |grep -x -e "-\{1,\}add" |tail -1 |sed -e "s/^-\{1,\}//")"
	materials_check=$(cat $materials |sed $'s/\t*$//g;s/ *$//g' |grep -e "$mat_sky$")
    if [ ! "$materials_check" ]; then echo ">> Error: The \"$mat_sky\" material definition was not found in the \"$materials\" file.";exit 0;fi;
fi
op="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -x -e "-\{1,\}op" |tail -1 |sed -e "s/^-\{1,\}//")"

# Make relevant directories..
mkdir -p $resdir $matdir $optdir $skydir $resdir_dat $resdir_ill $resdir_sd $resdir_add $tmpdir $logdir

# Log
#timestart=$(date +%F-%H%M%S)
timestart=$(date +%Y-%m-%d-%H%M%S)
log="$logdir/log_${timestart}.log"
echo "[$timestart]  Starting.." > $log

#dir="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
dir="$( cd "$( dirname "$0" )" && pwd )"
echo -e ">  Running.\n>  Use the following command in a new terminal to follow the log file live:\n      tail -f ${dir}/log/$(ls ${dir}/log |tail -1)"

# Lists for loops..
winlist=$(ls $windir |grep -i -e ".rad$")
ptslist=$(ls $ptsdir |grep -i -e ".pts$")
bsdflist=$(ls $matdir |grep -i -e ".xml$")
wealist=$(ls $weadir |grep -i -e ".epw$")

# Custom operation..
if [ $op ];then
    echo "Operating using input.csv..."
    
    exit;
fi

# Generate skies..
sky="./skies/sky_white_360.rad"
sky_rflux="./skies/sky_white_360_rflux.rad"
sky_blank="./skies/sky.rad"
echo -e "void glow sky_glow 0 0 4 1 1 1 0\nsky_glow source sky 0 0 4 0 0 1 360" > $sky
#echo -e "void glow ground_glow 0 0 4 1 1 1 0\nground_glow source sky 0 0 4 0 0 1 180\nvoid glow sky_glow 0 0 4 1 1 1 0\nsky_glow source sky 0 0 4 0 0 1 180" > $sky
echo -e "#@rfluxmtx h=u u=Y\nvoid glow ground_glow 0 0 4 1 1 1 0\nground_glow source sky 0 0 4 0 0 -1 180\n#@rfluxmtx h=r${granularity} u=Y\nvoid glow sky_glow 0 0 4 1 1 1 0\nsky_glow source sky 0 0 4 0 0 1 180" > $sky_rflux
echo -e "skyfunc glow sky_mat 0 0 4 1 1 1 0\nsky_mat source sky 0 0 4 0 0 1 180\nskyfunc glow ground_glow 0 0 4 1 .8 .5 0\nground_glow source ground 0 0 4 0 0 -1 180" > $sky_blank
echo "[$(date +%Y-%m-%d-%H%M%S)]  Sky files generated." >> $log

# Generate options files..
opt="$optdir/general.opt"
opt_vmx="$optdir/vmx.opt"
opt_dmx="$optdir/dmx.opt"
opt_sd="$optdir/solar.opt"
opt_sd_sp="$optdir/solar_sp.opt"
echo "-ab 6 -ad 4096 -as 1024 -ar 256 -aa 0.1 -lw 0.0003 -dt 0 -ds 0.1" > $opt
#echo "-ab 12 -ad 50000 -lw 2e-5" > $opt_vmx
#echo "-ab 9 -ad 16384 -lw 6.10e-5" > $opt_vmx
echo "-ab 6 -ad 4096 -as 1024 -ar 256 -aa 0.1 -lw 0.0003 -dt 0 -ds 0.1" > $opt_vmx
echo "-ab 6 -ad 4096 -as 1024 -ar 256 -aa 0.1 -lw 0.0003 -dt 0 -ds 0.1" > $opt_dmx
echo "-ab 0 -ad 512 -as 128 -ar 128 -aa 0.1" > $opt_sd
echo "-ab 1 -ad 4096 -as 1024 -ar 1024 -aa 0.08 -lw 0.0003 -dt 0" > $opt_sd_sp
echo "[$(date +%Y-%m-%d-%H%M%S)]  Options files generated." >> $log

# Define the light material for the view matrix and define the materials required for both the daylight and view matrices.
rm -f -r $dmx $vmx $solardisc
echo "void light window_vmx 0 0 3 1 1 1" >> $vmx
for win in $winlist; do
    winname=$(basename ${win} .${win#*.})
    # Modifiers must match the filenames of each window group file in ./objects/windows/
    echo "inherit alias $winname window_vmx" >> $vmx
    echo "inherit alias $winname $mat_glaz" >> $dmx
    echo "inherit alias $winname $mat_wall" >> $solardisc
done
echo "inherit alias nonoperated_glaz $mat_glaz" >> $vmx
echo "inherit alias nonoperated_glaz $mat_wall" >> $dmx
echo "inherit alias nonoperated_glaz $mat_wall" >> $solardisc
echo "inherit alias nonoperated_sky $mat_sky" >> $vmx
echo "inherit alias nonoperated_sky $mat_wall" >> $dmx
echo "inherit alias nonoperated_sky $mat_wall" >> $solardisc
echo "[$(date +%Y-%m-%d-%H%M%S)]  Materials assigned." >> $log

#oconv -w $materials $dmx $model $sky > ${modelname}_dmx.oct
#oconv -w $materials $vmx $model > ${modelname}_vmx.oct
#echo "[$(date +%Y-%m-%d-%H%M%S)]  OCTREES generated: ./${modelname}_dmx.oct ./${modelname}_vmx.oct" >> $log

model_rflux="./${modelname}_rflux.rad"

modlist=""

: <<'STOPONE'
for pts in $ptslist; do
	ptsname=$(basename ${pts} .${pts##*.})
	cat $ptsdir/$pts |sed -e "/^$/d" > $tmpdir/$pts
	echo "" >> $tmpdir/$pts
	
	for win in $winlist; do
		winname=$(basename ${win} .${win##*.})
		#win_o=$(echo $winname |sed -e "s/-/_/g;s/ /_/g;s/\./_/g" |rev |cut -d '_' -f1)
		#win_o=$(echo $winname |sed -e "s/-/_/g;s/ /_/g;s/\./_/g")
		#win_o=${win_o##*_}
		#echo ${win_o##*_}
		
		echo -e "#@rfluxmtx h=kf u=Z o=$resdir/%s_${ptsname}.vmx\n\n" > $tmpdir/rflux_${ptsname}_${winname}.rad
		cat $windir/$win >> $tmpdir/rflux_${ptsname}_${winname}.rad
		#echo -e "\n\ninherit alias glass01_s void" >> $tmpdir/rflux_${ptsname}_${winname}.rad
        
		#cat $model |sed -e "s/\!xform \.\/objects\/windows\/${winname}\.rad/\#\!xform \.\/objects\/windows\/${winname}\.rad/g" > $model_rflux
		cat ./$model |sed -e "s/^\!xform .*\/${winname}\.rad.$/\#\!xform \.\/objects\/windows\/${winname}\.rad/g" |sed -e "s/^\!xform \.\/materials\/materials\.rad.$/\#\!xform \.\/materials\/materials\.rad/g" > $model_rflux
		
		if [ -f $resdir/${winname}_${ptsname}.vmx ];then
			echo ">> Warning: Existing view matrix data found for: $resdir/${winname}_${ptsname}.vmx"
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Existing view matrix data found for: $resdir/${winname}_${ptsname}.vmx" >> $log
		else
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating view matrix for \"${winname}\" and \"${ptsname}\" analysis grid." >> $log
			#cat $tmpdir/$pts |rfluxmtx @$opt_vmx - $tmpdir/rflux_${ptsname}_${winname}.rad $materials $vmx $model
			rfluxmtx -faf -I+ @$opt_vmx -w < $tmpdir/$pts - $tmpdir/rflux_${ptsname}_${winname}.rad $materials $vmx $model_rflux
			echo "[$(date +%Y-%m-%d-%H%M%S)]  View matrices generated: $resdir/${winname}_${ptsname}.vmx" >> $log
		fi
		
		if [ -f $resdir/${winname}_m${granularity}.dmx ];then
			echo ">> Warning: Existing daylight matrix data found for: $resdir/${winname}_m${granularity}.dmx"
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Existing daylight matrix data found for: $resdir/${winname}_m${granularity}.dmx" >> $log
		else
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating daylight matrix for \"${winname}\" and \"${ptsname}\" with an \"MF:${granularity}\" setting." >> $log
			rfluxmtx -faf -w $tmpdir/rflux_${ptsname}_${winname}.rad $sky_rflux $materials $dmx $model_rflux > $resdir/${winname}_m${granularity}.dmx
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Daylight matrix generated: $resdir/${winname}_m${granularity}.dmx" >> $log
		fi
    done
	#echo -e "#@rfluxmtx h=kf u=Z o=./results/vmx/window_%03d.hdr\n\ninherit alias $winname window_vmx\n\n" > $tmpdir/rflux_${winname}.rad
	
	if [ $add ];then
		cat $solardisc > $tmpdir/$tmp_additional
		echo "inherit alias nonoperated_glaz $mat_glaz" >> $tmpdir/$tmp_additional
		echo "inherit alias nonoperated_sky $mat_sky" >> $tmpdir/$tmp_additional
		#oconv -w $materials $tmpdir/$tmp_additional $model $sky > ${modelname}_dmx_add.oct
		
		if [ -f $resdir/${modelname}_m${granularity}_add.dmx ];then
			echo ">> Warning: Existing daylight matrix data found for: $resdir/${modelname}_m${granularity}_add.dmx"
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Existing daylight matrix data found for: $resdir/${modelname}_m${granularity}_add.dmx" >> $log
		else
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating single-phase solardisc daylight matrix for single-phase non-BSDF light source contribution with an \"MF:${granularity}\" setting." >> $log
			#rfluxmtx -faf -I+ @$opt_dmx -w < $tmpdir/$pts - $sky_rflux $materials $tmpdir/$tmp_additional $model > $resdir/${modelname}_m${granularity}_add.dmx
			rfluxmtx -faf -I+ @$opt_dmx -w < $tmpdir/$pts - $sky_rflux $materials $tmpdir/$tmp_additional $model > $resdir/${modelname}_m${granularity}_add.dmx
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Single-phase non-BSDF light source contribution daylight matrix generated: $resdir/${modelname}_m${granularity}_add.dmx" >> $log
		fi
	fi
	
    #unknown=0
    # Modifiers must match the filenames of each window group file in ./objects/windows/
    #if [ $(echo $win_o |grep -i -e "s") ];then viewdir="0 -1 0";elif [ $(echo $win_o |grep -i -e "w") ];then viewdir="-1 0 0";elif [ $(echo $win_o |grep -i -e "n") ];then viewdir="0 1 0";elif [ $(echo $win_o |grep -i -e "e") ];then viewdir="1 0 0";else echo ">  Warning: "$winname" orientation unknown.";unknown=1;fi;
    #if [ $unknown -eq 0 ];then
        # Generate the daylight matrix..
    #    echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating daylight matrix for \"${winname}\" with an \"MF:${granularity}\" setting." >> $log
    #    #genklemsamp -vd $viewdir $windir/$win |rcontrib -fo -faf -c 1000 -e MF:$granularity -f reinhart.cal -o $resdir/${winname}_m${granularity}.dmx -b rbin -bn Nrbins -m sky_glow -w ${modelname}_dmx.oct
	#	perl "$genklemsamp" -vd $viewdir $windir/$win |rcontrib -fo -faf -c 1000 -e MF:$granularity -f reinhart.cal -o $resdir/${winname}_m${granularity}.dmx -b rbin -bn Nrbins -m sky_glow -w ${modelname}_dmx.oct
    #    echo "[$(date +%Y-%m-%d-%H%M%S)]  Daylight matrix generated: $resdir/${winname}_m${granularity}.dmx" >> $log
    #    modlist="${modlist} -b kbin$(echo $win_o |tr '[:lower:]' '[:upper:]') -m $winname"
    #fi
done

rm -f -r $model_rflux

# Generate the view matrices..
#if [ ! "$modlist" ];then echo ">> Error: Material modifiers could not be determined for the view matrix."; #exit 0;
#else
#    for pts in $ptslist; do
#        ptsname=$(basename ${pts} .${pts##*.})
#        cat $ptsdir/$pts |sed -e "/^$/d" > $tmpdir/$pts
#        echo "" >> $tmpdir/$pts
#        echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating view matrix for \"${ptsname}\" analysis grid." >> $log
#        cat $tmpdir/$pts |rcontrib -fo -faf -f klems_full.cal -bn Nkbins -o $resdir/${ptsname}_%s.vmx $modlist -I+ @${opt} -w ${modelname}_vmx.oct
#		echo "[$(date +%Y-%m-%d-%H%M%S)]  View matrices generated: $resdir/${ptsname}_*.vmx" >> $log
#    done
#fi
STOPONE
# Parse EPW weather file into WEA format for use with gendaymtx..
for wea in $wealist; do
    weafile="$weadir/$wea"
    weaname=$(basename $wea .${wea##*.})
    timestep="$weadir/${weaname}_timestep.dat"
    # Location is the first line of the EPW header..
    loc=$(head -1 $weafile)
    # Data period is the 8th line of the EPW header..
    dataper=$(head -8 $weafile |tail -1)
    lat=$(echo $loc |cut -d ',' -f7)
    longitude=$(echo $loc |cut -d ',' -f8)
    # EPW lists longitude + to the east, Radiance + to the west..
    lon=$(echo $longitude |rcalc -e '$1=-1*$1')
    meridian=$(echo $loc |cut -d ',' -f9)
    # EPW lists time zone in hours, but we need degrees..
    mer=$(echo $meridian |rcalc -e '$1=$1*-15')
    # WEA header format.. weather_data_file_units is either
    # field 2 (described as "#Number" in the EPW format documentation) or
    # field 3 (described as "#Number Records/Intervals in an hour" in the EPW format documentation).
    # [http://apps1.eere.energy.gov/buildings/energyplus/pdfs/weatherdatainformation.pdf]
    # This script uses field 2.
    echo -e "place $(echo $loc |cut -d "," -f2)_$(echo $loc |cut -d "," -f4)\nlatitude $lat\nlongitude $lon\ntime_zone $mer\nsite_elevation $(echo $loc |cut -d "," -f10)\nweather_data_file_units $(echo $dataper |cut -d "," -f2)" > "$weadir/${weaname}.wea"
    tail -n +9 $weafile |rcalc -t, -e '$1=$2;$2=$3;$3=$4;$4=$15;$5=$16' |sed -e "s/,/ /g" >> $weadir/$weaname.wea
    tail -n +7 $weadir/$weaname.wea |cut -d ' ' -f1-3 > $timestep
    # Generate sky vector matrix..
    #if [ -f $weadir/${weaname}--m${granularity}${rztag}.smx ];then echo ">> Warning: Existing sky vector matrix found: $weadir/${weaname}--m${granularity}${rztag}.smx";
    #else
    #    echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating sky vector matrix with an \"-m ${granularity}\" setting" >> $log
    #    gendaymtx -h -r $(awk -v rotation=$rotation 'BEGIN{print (rotation*-1);}') -m $granularity -of $weadir/$weaname.wea > "$weadir/${weaname}--m${granularity}${rztag}.smx"
    #    echo "[$(date +%Y-%m-%d-%H%M%S)]  Sky vector matrix generated: $weadir/${weaname}_m${granularity}${rztag}.smx" >> $log
    #fi
    # Generate direct sun sky vector matrix..
    #if [ -f $weadir/${weaname}--m${granularity}${rztag}--sol.smx ];then echo ">> Warning: Existing direct sun sky vector matrix found: $weadir/${weaname}--m${granularity}${rztag}--sol.smx";
    #elif [ $rcontrib ];then
    #else
    #    echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating direct sun sky vector matrix with an \"-m 4\" setting" >> $log
    #    gendaymtx -h -d -r $(awk -v rotation=$rotation 'BEGIN{print (rotation*-1);}') -m ${granularity} -of $weadir/$weaname.wea > "$weadir/${weaname}--m${granularity}${rztag}--sol.smx"
    #    echo "[$(date +%Y-%m-%d-%H%M%S)]  Direct sun sky vector matrix generated: $weadir/${weaname}--m${granularity}${rztag}--sol.smx" >> $log
    #fi
done

: <<'STOPONEHALF'
# Contribution from non-operated window openings
if [ $add ];then
    cat $solardisc > $tmpdir/$tmp_additional
    echo "inherit alias nonoperated_glaz $mat_glaz" >> $tmpdir/$tmp_additional
    echo "inherit alias nonoperated_sky $mat_sky" >> $tmpdir/$tmp_additional
    #echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating OCTREE for single-phase non-BSDF light source contribution." >> $log
    #oconv -w $materials $tmpdir/$tmp_additional $model $sky > ${modelname}_dmx_add.oct
    for pts in $ptslist; do
        ptsname=$(basename $pts .${pts##*.})
		pts_nrows=$(cat $tmpdir/$pts |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
        #echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating single-phase solardisc daylight matrix for single-phase non-BSDF light source contribution with an \"MF:${granularity}\" setting." >> $log
        #rcontrib -fo -faf -I+ -e MF:${granularity} -f reinhart.cal -o $resdir/${modelname}_m${granularity}_add.dmx -b rbin -bn Nrbins -m sky_glow @${opt_vmx} -w ${modelname}_dmx_add.oct < $tmpdir/$pts
		#echo "[$(date +%Y-%m-%d-%H%M%S)]  Single-phase non-BSDF light source contribution daylight matrix generated: $resdir/${modelname}_m${granularity}_add.dmx" >> $log
        for wea in $wealist; do
            weaname=$(basename $wea .${wea##*.})
            outname="${ptsname}--${weaname}--m${granularity}${rztag}"
            timestep="$weadir/${weaname}_timestep.dat"
			
			if [ -f $resdir_add/${outname}.dat ];then
				nrows=$(cat $resdir_add/${outname}.dat |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
				#ncols=$(head -1 $resdir_add/${outname}.dat |sed -e "s/[ ]*.$//g" |tr ' ' '\n' |wc -l |awk '{printf "%.0f", $1}') # IMPORTANT Windows
				ncols=$(head -1 $resdir_add/${outname}.dat |tr ' ' '\n' |wc -l |awk '{printf "%.0f", $1}') # IMPORTANT Linux
				echo "$nrows (r) x $ncols (c)"
				if [ $nrows -eq $pts_nrows ] && [ $ncols -eq 8760 ];then
					echo ">> Warning: Existing dctimestep data found for: $resdir_add/${outname}.dat"
					echo "[$(date +%Y-%m-%d-%H%M%S)]  Existing dctimestep data found for: $resdir_add/${outname}.dat" >> $log
					continue
				fi
			fi
			
            #echo -n "" > "$resdir_add/${outname}.ill"
            echo "[$(date +%Y-%m-%d-%H%M%S)]  Running single-phase non-BSDF light source contribution dctimestep on: $resdir/${modelname}_m${granularity}_add.dmx $weadir/${weaname}--m${granularity}${rztag}.smx" >> $log
            #save_IFS=$IFS
            #dctimestep -n 8760 -if $resdir/${modelname}_m${granularity}_add.dmx $weadir/${weaname}--m${granularity}${rztag}.smx |sed -e '/^$/d' |tail -n +9 |sed -e "s/^\t//g;s/^ //g;s/\t/#/g" |while read line; do IFS='#';echo $line |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//" >> "$resdir_add/${outname}.dat";done;
			
			#dctimestep -n 8760 -if $resdir/${modelname}_${ptsname}_m${granularity}_add.dmx $weadir/${weaname}--m${granularity}${rztag}.smx |sed -e '/^$/d' |tail -n +9 |tr '\t' '#' |sed -e "s/^#//g;s/^ //g;s/^$//g" > ./temp/temp_dc.dat
			#while read line; do
			#	IFS='#'
			#	echo "$line" |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ,conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//" >> "$resdir_add/${outname}.dat"
			#done < "./temp/temp_dc.dat"
			
			#IFS='#'
			#cat ./temp/temp_dc.dat |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ,conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//" >> "$resdir_add/${outname}.dat"
			
			#dctimestep -n 8760 -if $resdir/${modelname}_m${granularity}_add.dmx $weadir/${weaname}--m${granularity}${rztag}.smx |sed -e '/^$/d' |tail -n +9 |tr '\t' '#' |sed -e "s/^#//g;s/^ //g" |while read line; do IFS='#';echo $line |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//" >> "$resdir_add/${outname}.dat";done;
            #IFS=$save_IFS
			
			#echo -n "" > $tmpdir/$tmp_dat
			#echo -n "" > "$resdir_add/${outname}.dat"
			#dctimestep -n 8760 -if $resdir/${modelname}_m${granularity}_add.dmx $weadir/${weaname}--m${granularity}${rztag}.smx |sed -e '/^$/d' |tail -n +10 |tr '\t' '#' >> $tmpdir/$tmp_dat
			#while read line;do
			#	echo $line |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//;s/ \+/#/g" |tr '#' ' ' >> "$resdir_add/${outname}.dat"
			#done < $tmpdir/$tmp_dat
			
			echo -n "" > $tmpdir/$tmp_dat
			echo -n "" > "$resdir_add/${outname}.dat"
			dctimestep -n 8760 -if $resdir/${modelname}_m${granularity}_add.dmx $weadir/${weaname}--m${granularity}${rztag}.smx |sed -e '/^$/d' |tail -n +9 |tr '\t' '#' >> $tmpdir/$tmp_dat # IMPORTANT tail -n +9 for Linux, tail -n +10 for Windows...
			while read line;do
				echo $line |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//;s/ \+/#/g" |tr '#' ' ' >> "$resdir_add/${outname}.dat"
			done < $tmpdir/$tmp_dat
			
            echo "[$(date +%Y-%m-%d-%H%M%S)]  Single-phase non-BSDF light source contribution dctimestep finished: $resdir_add/${outname}.dat" >> $log
        done
    done
fi


# Pull it all together with dctimestep..
for wea in $wealist; do

    weaname=$(basename $wea .${wea##*.})
    timestep="$weadir/${weaname}_timestep.dat"
    for pts in $ptslist; do

        ptsname=$(basename ${pts} .${pts##*.})
		pts_nrows=$(cat $tmpdir/$pts |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
        for bsdf in $bsdflist; do

            bsdfname=$(basename $bsdf .${bsdf##*.})
            for win in $winlist; do
                winname=$(basename ${win} .${win##*.})
                    outname="${ptsname}--${winname}--${bsdfname}--${weaname}--m${granularity}${rztag}"
					
					if [ -f $resdir_dat/${outname}.dat ];then
						nrows=$(cat $resdir_dat/${outname}.dat |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
						#ncols=$(head -1 $resdir_dat/${outname}.dat |sed -e "s/[ ]*.$//g" |tr ' ' '\n' |wc -l |awk '{printf "%.0f", $1}') # IMPORTANT Windows
						ncols=$(head -1 $resdir_dat/${outname}.dat |tr ' ' '\n' |wc -l |awk '{printf "%.0f", $1}') # IMPORTANT Linux
						if [ $nrows -eq $pts_nrows ] && [ $ncols -eq 8760 ];then
							echo ">> Warning: Existing dctimestep data found for: $resdir_dat/${outname}.dat"
							echo "[$(date +%Y-%m-%d-%H%M%S)]  Existing dctimestep data found for: $resdir_dat/${outname}.dat" >> $log
							continue
						fi
					fi
					
                    echo "[$(date +%Y-%m-%d-%H%M%S)]  Running dctimestep on: $resdir/${winname}_${ptsname}.vmx $matdir/$bsdf $resdir/${winname}_m${granularity}.dmx $weadir/${weaname}--m${granularity}${rztag}.smx" >> $log
                    #save_IFS=$IFS
                    #dctimestep -n 8760 -if $resdir/${ptsname}_${winname}.vmx $matdir/$bsdf $resdir/${winname}_m${granularity}.dmx $weadir/${weaname}--m${granularity}${rztag}.smx |sed -e '/^$/d' |tail -n +9 |sed -e "s/^\t//g;s/^ //g;s/\t/#/g" |while read line; do IFS='#';echo $line |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//;s/nan/0/g" >> "$resdir_dat/${outname}.dat";done;
                    #IFS=$save_IFS
                    
					#echo -n "" > $tmpdir/$tmp_dat
					#echo -n "" > "$resdir_dat/${outname}.dat"
					#dctimestep -n 8760 -if $resdir/${ptsname}_${winname}.vmx $matdir/$bsdf $resdir/${winname}_m${granularity}.dmx $weadir/${weaname}--m${granularity}${rztag}.smx |sed -e '/^$/d' |tail -n +10 |tr '\t' '#' >> $tmpdir/$tmp_dat
					#while read line;do
					#	echo $line |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//;s/ \+/#/g" |tr '#' ' ' >> "$resdir_dat/${outname}.dat"
					#done < $tmpdir/$tmp_dat
					#echo "[$(date +%Y-%m-%d-%H%M%S)]  dctimestep finished: $resdir_dat/${outname}.dat" >> $log
					
					echo -n "" > $tmpdir/$tmp_dat
					echo -n "" > "$resdir_dat/${outname}.dat"
					dctimestep -n 8760 -if $resdir/${winname}_${ptsname}.vmx $matdir/$bsdf $resdir/${winname}_m${granularity}.dmx $weadir/${weaname}--m${granularity}${rztag}.smx |sed -e '/^$/d' |tail -n +9 |tr '\t' '#' >> $tmpdir/$tmp_dat # IMPORTANT tail -n +9 for Linux, tail -n +10 for Windows...
					while read line;do
						echo $line |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//;s/ \+/#/g" |tr '#' ' ' >> "$resdir_dat/${outname}.dat"
					done < $tmpdir/$tmp_dat
					echo "[$(date +%Y-%m-%d-%H%M%S)]  dctimestep finished: $resdir_dat/${outname}.dat" >> $log
                    
                    #if [ $add ];then
                    #    echo "[$(date +%Y-%m-%d-%H%M%S)]  Adding contribution of non-BSDF sources to $resdir_dat/${outname}.dat" >> $log
                        #awk '
                        #FNR==NR {
                        #for(i=1; i<=NF; i++)
                        #    _[FNR,i]=$i
                        #    next
                        #}
                        #{
                        #for(i=1; i<=NF; i++)
                        #    printf("%d%s", $i+_[FNR,i], (i==NF) ? "\n" : FS);
                        #}' $resdir_dat/${outname}.dat $resdir_add/${ptsname}--${weaname}--m${granularity}${rztag}.dat > $tmpdir/${outname}.dat
                    #    res_cnum=$(head -1 $resdir_dat/${outname}.dat |tr ' ' '\n' |wc -l)
                    #    res_cnum_add=$(head -1 $resdir_add/${ptsname}--${weaname}--m${granularity}${rztag}.dat |tr ' ' '\n' |wc -l)
                    #    export res_cnum;
                    #    paste $resdir_dat/${outname}.dat $resdir_add/${ptsname}--${weaname}--m${granularity}${rztag}.dat |perl -alne 'print join " ", map {$F[$_] + $F[$_+$ENV{res_cnum}]} 0..$ENV{res_cnum}-1' > $tmpdir/${outname}.dat
                    #    if [ "$res_cnum" != "$res_cnum_add" ];then
                    #        echo "[$(date +%Y-%m-%d-%H%M%S)] Warning: Number of columns do not match: $resdir_dat/${outname}.dat [$res_cnum] and $resdir_add/${ptsname}--${weaname}--m${granularity}${rztag}.dat [$res_cnum_add]" >> $log
                    #    fi
                    #    echo "[$(date +%Y-%m-%d-%H%M%S)]  Non-BSDF sources added to $resdir_dat/${outname}.dat" >> $log
                        
                    #    mv $resdir_dat/${outname}.dat $resdir_dat/${outname}_orig.dat
                        #rm $resdir_dat/${outname}.dat
                    #    mv $tmpdir/${outname}.dat $resdir_dat/${outname}.dat
                    #fi
            done
        done
    done
done

echo "" >> $log

#count_me=0
STOPONEHALF
# Solar disc analysis..
echo "[$(date +%Y-%m-%d-%H%M%S)]  Starting solardisc analysis.." >> $log
if [ ! $rcontrib ];then
    # Rtrace solar disc analysis..
    wealist=$(ls $weadir |grep ".wea$")
    for pts in $ptslist; do
        ptsname=$(basename ${pts} .${pts##*.})
	echo "PTS: $ptsname"
	cat $ptsdir/$pts |sed -e "/^$/d" > $tmpdir/$pts
	echo "" >> $tmpdir/$pts
        npts=$(cat $tmpdir/$pts |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
        nullvec=""
        for i in $(seq 1 $npts); do nullvec="${nullvec}0#";done;
        #for win in $winlist; do
            #winname=$(basename $win .${win##*.})
	    winname="all"
            #cat $solardisc > $tmpdir/$tmp_solardisc
            #echo "inherit alias $winname $mat_glaz" >> $tmpdir/$tmp_solardisc
            for wea in $wealist; do
                weaname=$(basename $wea .${wea##*.})
		echo "WEA: $weaname"
				#if [ -f $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill ];then
				#	nrows=$(cat $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
				#	if [ $nrows -eq 8760 ];then
				#		echo ">> Warning: Existing solar disc analysis data found for: $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill"
				#		echo "[$(date +%Y-%m-%d-%H%M%S)]  Existing solar disc analysis data found for: $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill" >> $log
				#		cat $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill |tr '#' ' ' |rcollate -h -fa1 -t |tr '\t' ' ' > $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.dat
				#		continue
				#	fi
				#fi
				
                tail -n +7 $weadir/$wea > $tmpdir/$tmp_wea
                echo -n "" > $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill
                echo "[$(date +%Y-%m-%d-%H%M%S)]  Starting solar disc analysis for \"${modelname}_${winname}\"" >> $log
                while read line; do
                    month=$(echo $line |cut -d ' ' -f1)
                    day=$(echo $line |cut -d ' ' -f2)
                    time=$(echo $line |cut -d ' ' -f3)
                    irr_dir=$(echo $line |cut -d ' ' -f4)
                    irr_diff=$(echo $line |cut -d ' ' -f5)
                    lat=$(head -2 $weadir/$wea |tail -1 |cut -d ' ' -f2)
                    lon=$(head -3 $weadir/$wea |tail -1 |cut -d ' ' -f2)
                    mer=$(head -4 $weadir/$wea |tail -1 |cut -d ' ' -f2)
                    if [ $irr_dir -eq 0 ] && [ $irr_diff -eq 0 ];then
                        echo "$nullvec" >> $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill
                        echo -n "" > $tmpdir/$tmp_sky
                        echo -e "$month\t$day\t$time\t  | $irr_dir | $irr_diff | EMPTY" >> $log
                    else
                        warnchk=""
                        warnchk=$(gendaylit $month $day $time -a $lat -o $lon -m $mer -W $irr_dir $irr_diff 2>&1 |head -2 |grep -i "range warning")
                        if [ ! "$warnchk" ];then
                            echo "!gendaylit $month $day $time -a $lat -o $lon -m $mer -W $irr_dir $irr_diff -w |xform -rz $(awk -v rotation=$rotation 'BEGIN{print (rotation*-1);}')" > $tmpdir/$tmp_sky
                            echo "" >> $tmpdir/$tmp_sky
                            cat $sky_blank >> $tmpdir/$tmp_sky
                            echo -e "$month\t$day\t$time\t  | $irr_dir | $irr_diff | GENDAYLIT" >> $log
                        elif [ $irr_dir -eq 0 ];then
                            echo -n "" > $tmpdir/$tmp_sky
                            echo -e "$month\t$day\t$time\t  | $irr_dir | $irr_diff | EMPTY (dir_nor=0)" >> $log
                        else
                            echo "!gensky $month $day $time +s -a $lat -o $lon -m $mer |xform -rz $(awk -v rotation=$rotation 'BEGIN{print (rotation*-1);}')" > $tmpdir/$tmp_sky
                            echo "" >> $tmpdir/$tmp_sky
                            cat $sky_blank >> $tmpdir/$tmp_sky
                            echo -e "$month\t$day\t$time\t  | $irr_dir | $irr_diff | GENSKY" >> $log
                        fi
                        #oconv -w $materials $tmpdir/$tmp_solardisc $model $tmpdir/$tmp_sky > $tmpdir/${modelname}_${winname}.oct
			oconv -w $materials $model $tmpdir/$tmp_sky > $tmpdir/${modelname}_${winname}.oct
                        #rtrace -I -h -w @${opt_sd} $tmpdir/${modelname}_${winname}.oct < $tmpdir/$pts |rcalc -e '$1=179*($1*0.265+$2*0.670+$3*0.065)' |tr '\n' ' ' >> $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill
						rtrace -I -h -w @${opt_sd} $tmpdir/${modelname}_${winname}.oct < $tmpdir/$pts |tr '\n' '#' |tr '\t' ' ' |sed -e "s/#\+/#/g;s/ #/#/g;s/#$//g" |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//;s/ \+/#/g" >> $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill
						
						#rtrace -I -h -w @./options/solar.opt ./model_dmx.oct < ./temp/points.pts |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//;s/ \+/#/g"
                        # ^^^ |sed -e "s/$/\n/"
						#rtrace -I -h -w @${opt_sd} $tmpdir/${modelname}_${winname}.oct < $tmpdir/$pts |rcalc -e '$1=179*($1*0.265+$2*0.670+$3*0.065)' |awk '{if($1==0){printf("%.0f ",$1)}else{printf("%.2f ",$1)}}' |sed -e "s/$/\n/;s/^/$month $day $time /" >> $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill
                    fi
					#count_me=$((count_me+1))
					#if [ $count_me -gt 18 ];then
					#	exit;
					#fi
                done < "$tmpdir/$tmp_wea"
                # Transpose rtrace results..
                echo "[$(date +%Y-%m-%d-%H%M%S)]  Transposing \"$resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill\"" >> $log
                #rcollate -h -fa1 -t $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill |tr '#' ' ' |tr '\t' '\n' > $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.dat
				cat $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill |tr '#' ' ' |rcollate -h -fa1 -t |tr '\t' ' ' > $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.dat
                echo "[$(date +%Y-%m-%d-%H%M%S)]  Transpose finished: $resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.ill" >> $log
            done
        #done
    done
else
    # Single-phase rcontrib solar disc analysis..
    wealist=$(ls $weadir |grep ".wea$")
    for pts in $ptslist; do
        ptsname=$(basename ${pts} .${pts##*.})
        npts=$(cat $tmpdir/$pts |sed -e "/^$/d" |wc -l)
        nullvec=""
        for i in $(seq 1 $npts); do nullvec="${nullvec}0 ";done;
        for win in $winlist; do
            winname=$(basename $win .${win##*.})
            # Generate single-phase daylight matrix for solardisc analysis..
            cat $solardisc > $tmpdir/$tmp_solardisc
            echo "inherit alias $winname $mat_glaz" >> $tmpdir/$tmp_solardisc
            echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating OCTREE for single-phase solardisc analysis for \"${winname}\"." >> $log
            oconv -w $materials $tmpdir/$tmp_solardisc $model $sky > ${modelname}_dmx_sol.oct
            echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating single-phase solardisc daylight matrix for \"${winname}\" with an \"MF:4\" setting." >> $log
            rcontrib -fo -faf -I+ -e MF:4 -f reinhart.cal -o $resdir/${winname}_m4_sol.dmx -b rbin -bn Nrbins -m sky_glow @${opt_sd_sp} -w ${modelname}_dmx_sol.oct < $tmpdir/$pts
            echo "[$(date +%Y-%m-%d-%H%M%S)]  Single-phase solardisc daylight matrix generated: $resdir/${winname}_m4_sol.dmx" >> $log
            
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating single-phase solardisc daylight matrix for single-phase non-BSDF light source contribution with an \"MF:${granularity}\" setting." >> $log
			rfluxmtx -faf -I+ @$opt_vmx -w < $tmpdir/$pts - $sky_rflux $materials $tmpdir/$tmp_solardisc $model > $resdir/${winname}_m${granularity}_sol.dmx
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Single-phase non-BSDF light source contribution daylight matrix generated: $resdir/${modelname}_m${granularity}_add.dmx" >> $log
			
            for wea in $wealist; do
                weaname=$(basename $wea .${wea##*.})
                outname="${ptsname}--${winname}--${weaname}--m4${rztag}"
                timestep="$weadir/${weaname}_timestep.dat"
                echo -n "" > "$resdir_sd/${outname}.ill"
                echo -n "" > "$resdir_sd/${outname}.dat"
                echo "[$(date +%Y-%m-%d-%H%M%S)]  Running single-phase solardisc dctimestep on: $resdir/${winname}_m4_sol.dmx $weadir/${weaname}--m4${rztag}--sol.smx" >> $log
                #save_IFS=$IFS
                #dctimestep -n 8760 -if $resdir/${winname}_m4_sol.dmx $weadir/${weaname}--m4${rztag}--sol.smx |sed -e '/^$/d' |tail -n +9 |sed -e "s/^\t//g;s/^ //g;s/\t/#/g" |while read line; do IFS='#';echo $line |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//" >> "$resdir_sd/${outname}.dat";done;
                #IFS=$save_IFS
				
				#echo -n "" > $tmpdir/$tmp_dat
				#echo -n "" > "$resdir_sd/${outname}.dat"
				#dctimestep -n 8760 -if $resdir/${winname}_m4_sol.dmx $weadir/${weaname}--m4${rztag}--sol.smx |sed -e '/^$/d' |tail -n +10 |tr '\t' '#' >> $tmpdir/$tmp_dat
				#while read line;do
				#	echo $line |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//;s/ \+/#/g" >> "$resdir_sd/${outname}.dat"
				#done < $tmpdir/$tmp_dat
				#echo "[$(date +%Y-%m-%d-%H%M%S)]  dctimestep finished: $resdir_dat/${outname}.dat" >> $log
				
				echo -n "" > $tmpdir/$tmp_dat
				echo -n "" > "$resdir_sd/${outname}.dat"
				dctimestep -n 8760 -if $resdir/${winname}_m${granularity}_sol.dmx $weadir/${weaname}--m4${rztag}--sol.smx |sed -e '/^$/d' |tail -n +9 |tr '\t' '#' >> $tmpdir/$tmp_dat # IMPORTANT tail -n +9 for Linux, tail -n +10 for Windows...
				while read line;do
					echo $line |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//;s/ \+/#/g" >> "$resdir_sd/${outname}.dat"
				done < $tmpdir/$tmp_dat
				echo "[$(date +%Y-%m-%d-%H%M%S)]  dctimestep finished: $resdir_dat/${outname}.dat" >> $log
				
                echo "[$(date +%Y-%m-%d-%H%M%S)]  Single-phase solardisc dctimestep finished: $resdir_sd/${outname}.dat" >> $log
            done
        done
    done
fi
: <<'STOPTWO'
if [ $add ];then
    # Rtrace solar disc analysis for non-operated windows..
    wealist=$(ls $weadir |grep ".wea$")
    for pts in $ptslist; do
        ptsname=$(basename ${pts} .${pts##*.})
        npts=$(cat $tmpdir/$pts |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
        nullvec=""
        for i in $(seq 1 $npts); do nullvec="${nullvec}0#";done;
        for wea in $wealist; do
            weaname=$(basename $wea .${wea##*.})
			
			if [ -f $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill ];then
				nrows=$(cat $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
				if [ $nrows -eq 8760 ];then
					echo ">> Warning: Existing solar disc analysis data found for: $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill"
					echo "[$(date +%Y-%m-%d-%H%M%S)]  Existing solar disc analysis data found for: $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill" >> $log
					cat $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill |tr '#' ' ' |rcollate -h -fa1 -t |tr '\t' ' ' > $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.dat
					continue
				fi
			fi
			
            tail -n +7 $weadir/$wea > $tmpdir/$tmp_wea
            echo -n "" > $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill
            echo "[$(date +%Y-%m-%d-%H%M%S)]  Starting solar disc analysis for \"${modelname}_${ptsname}_nonoperated\"" >> $log
            while read line; do
                month=$(echo $line |cut -d ' ' -f1)
                day=$(echo $line |cut -d ' ' -f2)
                time=$(echo $line |cut -d ' ' -f3)
                irr_dir=$(echo $line |cut -d ' ' -f4)
                irr_diff=$(echo $line |cut -d ' ' -f5)
                lat=$(head -2 $weadir/$wea |tail -1 |cut -d ' ' -f2)
                lon=$(head -3 $weadir/$wea |tail -1 |cut -d ' ' -f2)
                mer=$(head -4 $weadir/$wea |tail -1 |cut -d ' ' -f2)
                if [ $irr_dir -eq 0 ] && [ $irr_diff -eq 0 ];then
                    echo "$nullvec" >> $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill
                    echo -n "" > $tmpdir/$tmp_sky
                    echo -e "$month\t$day\t$time\t  | $irr_dir | $irr_diff | EMPTY" >> $log
                else
                    warnchk=""
                    warnchk=$(gendaylit $month $day $time -a $lat -o $lon -m $mer -W $irr_dir $irr_diff 2>&1 |head -2 |grep -i "range warning")
                    if [ ! "$warnchk" ];then
                        echo "!gendaylit $month $day $time -a $lat -o $lon -m $mer -W $irr_dir $irr_diff -w |xform -rz $(awk -v rotation=$rotation 'BEGIN{print (rotation*-1);}')" > $tmpdir/$tmp_sky
                        echo "" >> $tmpdir/$tmp_sky
                        cat $sky_blank >> $tmpdir/$tmp_sky
                        echo -e "$month\t$day\t$time\t  | $irr_dir | $irr_diff | GENDAYLIT" >> $log
                    elif [ $irr_dir -eq 0 ];then
                        echo -n "" > $tmpdir/$tmp_sky
                        echo -e "$month\t$day\t$time\t  | $irr_dir | $irr_diff | EMPTY (dir_nor=0)" >> $log
                    else
                        echo "!gensky $month $day $time +s -a $lat -o $lon -m $mer |xform -rz $(awk -v rotation=$rotation 'BEGIN{print (rotation*-1);}')" > $tmpdir/$tmp_sky
                        echo "" >> $tmpdir/$tmp_sky
                        cat $sky_blank >> $tmpdir/$tmp_sky
                        echo -e "$month\t$day\t$time\t  | $irr_dir | $irr_diff | GENSKY" >> $log
                    fi
                    oconv -w $materials $tmpdir/$tmp_additional $model $tmpdir/$tmp_sky > $tmpdir/${modelname}_nonoperated.oct
                    #rtrace -I -h -w @${opt_sd} $tmpdir/${modelname}_nonoperated.oct < $tmpdir/$pts |rcalc -e '$1=179*($1*0.265+$2*0.670+$3*0.065)' |tr '\n' ' ' |sed -e "s/$/\n/" >> $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill
					rtrace -I -h -w @${opt_sd} $tmpdir/${modelname}_nonoperated.oct < $tmpdir/$pts |tr '\n' '#' |tr '\t' ' ' |sed -e "s/#\+/#/g;s/ #/#/g;s/#$//g" |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//;s/ \+/#/g" >> $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill
                fi
            done < "$tmpdir/$tmp_wea"
			# Transpose rtrace results..
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Transposing \"$resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill\"" >> $log
			#rcollate -h -fa1 -t $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill |tr '#' ' ' |tr '\t' '\n' > $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.dat
			cat $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill |tr '#' ' ' |rcollate -h -fa1 -t |tr '\t' ' ' > $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.dat
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Transpose finished: $resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.ill" >> $log
        done
    done
fi

echo "" >> $log



#echo "[$(date +%Y-%m-%d-%H%M%S)]  Starting blinds operation.." >> $log

# Blinds operation..
delimiter=" "
tmpmerge="$tmpdir/merge_tmp.ill"
#tmpmerge_rflux="$tmpdir/merge_tmp_rflux.ill"
tmpmerge_o="$tmpdir/merge_tmp_o.ill"
#tmpmerge_o_rflux="$tmpdir/merge_tmp_o_rflux.ill"
tmpmerge_c="$tmpdir/merge_tmp_c.ill"
#tmpmerge_c_rflux="$tmpdir/merge_tmp_c_rflux.ill"
tmpmerge_sd="$tmpdir/merge_tmp_sd.ill"
tmpstep="$tmpdir/sda_tmpstep.ill"
tmpda="$tmpdir/da_tmp.ill"
tmp2per="$tmpdir/2per_tmp.ill"
tmp2per_sort="$tmpdir/2per_tmp_sort.ill"
outsda="$resdir/sda.ill"
outlm83="$resdir/LM83_metrics.dat"
tmpgrid="$tmpdir/tmpgrid.ill"
tmptimestep="$tmpdir/tmptimestep.dat"

rm -f $tmpmerge $tmpmerge_o $tmpmerge_c $outsda $outlm83
#rm -f $tmpmerge_rflux $tmpmerge_o_rflux $tmpmerge_c_rflux

#IFS=$' \t\n'

# Rediefine starting and ending period..
#period_start=1
#period_end=24

# Loop over each weather file...
for wea in $wealist; do
    weaname=$(basename $wea .${wea##*.})
    timestep="$weadir/${weaname}_timestep.dat"
    for pts in $ptslist; do
        ptsname=$(basename ${pts} .${pts##*.})
        
		ave_x=$(cat $tmpdir/$pts |cut -d' ' -f1 |tr ' ' '\n' |awk '{x+=$1;x_cnt+=1}END{printf("%.6f",x/x_cnt)}')
		max_x=$(cat $tmpdir/$pts |cut -d' ' -f1 |tr ' ' '\n' |awk 'NR==1{x_max=$1}{if($1>x_max)x_max=$1}END{printf("%.6f",x_max)}')
		ave_y=$(cat $tmpdir/$pts |cut -d' ' -f2 |tr ' ' '\n' |awk '{y+=$1;y_cnt+=1}END{printf("%.6f",y/y_cnt)}')
		max_y=$(cat $tmpdir/$pts |cut -d' ' -f2 |tr ' ' '\n' |awk 'NR==1{y_max=$1}{if($1>y_max)y_max=$1}END{printf("%.6f",y_max)}')
		ave_z=$(cat $tmpdir/$pts |cut -d' ' -f3 |tr ' ' '\n' |awk '{z+=$1;z_cnt+=1}END{printf("%.6f",z/z_cnt)}')
		max_z=$(cat $tmpdir/$pts |cut -d' ' -f3 |tr ' ' '\n' |awk 'NR==1{z_max=$1}{if($1>z_max)z_max=$1}END{printf("%.6f",z_max)}')
		
		#if [ $(echo "scale=6;1-(${ave_y}/${max_y}) < 0.01" |bc -l) -eq 1 ] && [ $(echo "scale=6;1-(${ave_z}/${max_z}) > 0.01" |bc -l) -eq 1 ];then
		if [ $(echo "scale=6;if((${ave_y}/${max_y}) <= 1) 1-(${ave_y}/${max_y}) < 0.01 else (${ave_y}/${max_y})-1 < 0.01" |bc -l) -eq 1 ] && [ $(echo "scale=6;if((${ave_z}/${max_z}) <= 1) 1-(${ave_z}/${max_z}) > 0.01 else (${ave_z}/${max_z})-1 > 0.01" |bc -l) -eq 1 ];then
			cat $tmpdir/$pts |cut -d' ' -f1 |tr "\\r\\n" " " |sed -e "s/[ ]\+/ /g;s/ $//;s/^/   /" > $tmpgrid # |cut -d " " -f1 |tr "\\r\\n" " " |sed -e "s/ \+/ /g"
			echo "" >> $tmpgrid
			cat $tmpdir/$pts |cut -d' ' -f3 |tr "\\r\\n" " " |sed -e "s/[ ]\+/ /g;s/ $//;s/^/   /" >> $tmpgrid
			echo "" >> $tmpgrid
			#echo "> X-axis surface normal.. ${ptsname} | x: $ave_x/$max_x | y: $ave_y/$max_y | z: $ave_z/$max_z"
		#elif [ $(echo "scale=6;1-(${ave_x}/${max_x}) < 0.01" |bc -l) -eq 1 ] && [ $(echo "scale=6;1-(${ave_z}/${max_z}) < 0.01" |bc -l) -eq 1 ];then
		elif [ $(echo "scale=6;if((${ave_x}/${max_x}) <= 1) 1-(${ave_x}/${max_x}) < 0.01 else (${ave_x}/${max_x})-1 < 0.01" |bc -l) -eq 1 ] && [ $(echo "scale=6;if((${ave_z}/${max_z}) <= 1) 1-(${ave_z}/${max_z}) > 0.01 else (${ave_z}/${max_z})-1 > 0.01" |bc -l) -eq 1 ];then
			cat $tmpdir/$pts |cut -d' ' -f2 |tr "\\r\\n" " " |sed -e "s/[ ]\+/ /g;s/ $//;s/^/   /" > $tmpgrid # |cut -d " " -f1 |tr "\\r\\n" " " |sed -e "s/ \+/ /g"
			echo "" >> $tmpgrid
			cat $tmpdir/$pts |cut -d' ' -f3 |tr "\\r\\n" " " |sed -e "s/[ ]\+/ /g;s/ $//;s/^/   /" >> $tmpgrid
			echo "" >> $tmpgrid
			#echo "> Y-axis surface normal.. ${ptsname} | x: $ave_x/$max_x | y: $ave_y/$max_y | z: $ave_z/$max_z"
		else
			cat $tmpdir/$pts |cut -d' ' -f1 |tr "\\r\\n" " " |sed -e "s/[ ]\+/ /g;s/ $//;s/^/   /" > $tmpgrid # |cut -d " " -f1 |tr "\\r\\n" " " |sed -e "s/ \+/ /g"
			echo "" >> $tmpgrid
			cat $tmpdir/$pts |cut -d' ' -f2 |tr "\\r\\n" " " |sed -e "s/[ ]\+/ /g;s/ $//;s/^/   /" >> $tmpgrid
			echo "" >> $tmpgrid
			#echo "> Z-axis surface normal.. ${ptsname} | x: $ave_x/$max_x | y: $ave_y/$max_y | z: $ave_z/$max_z"
		fi
		#cat $tmpdir/$tmp_ill |perl -anF'#|\n' -e'$n=@F-1if!$n;for(0..$n){push@{$$m[$_]},$F[$_]}END{print map{join"#",@$_,"\n"}@$m}' |tr "#" " " |paste -d " " $timestep - |sed -e "s/ .$//g" >> $resdir/${outname}.ill
		
        # Final output files..
        outmerge="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--final.ill"
        #outmerge_rflux="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--final_rflux.ill"
        outda="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--da.ill"
        #outda_rflux="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--da_rflux.ill"
        outda_o="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--da_open.ill"
        #outda_o_rflux="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--da_open_rflux.ill"
        outda_c="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--da_closed.ill"
        #outda_c_rflux="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--da_closed_rflux.ill"
        outase_hours="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--ase_hours.ill"
        outmerge_o="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--${bsdf_open}.ill"
        #outmerge_o_rflux="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--${bsdf_open}_rflux.ill"
        outmerge_c="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--${bsdf_closed}.ill"
        #outmerge_c_rflux="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--${bsdf_closed}_rflux.ill"
        outmerge_sd="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--solardisc.ill"
        outsched="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--final_blinds_sched.dat"

		if [ -f $outmerge ];then
			nrows=$(tail -n+3 $outmerge |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
			nrows_o=$(tail -n+3 $outmerge_o |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
			nrows_c=$(tail -n+3 $outmerge_c |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
			nrows_sd=$(tail -n+3 $outmerge_sd |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
			nrows_sch=$(tail -n+2 $outsched |sed -e "/^$/d" |wc -l |awk '{printf "%.0f", $1}')
			nrows_small=$(echo $nrows $nrows_o $nrows_c $nrows_sd $nrows_sch |tr ' ' '\n' |sort -rn |sed -e "/^$/d" |tail -1)
			
			echo ">> Warning: Existing blinds operation results files found! ($nrows_small rows of data available, headers not counted)"
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Warning: Existing blinds operation files found! ($nrows_small rows of data available)" >> $log
			
			if [ $nrows -eq $nrows_o ] && [ $nrows -eq $nrows_c ] && [ $nrows -eq $nrows_sd ] && [ $nrows -eq $nrows_sch ];then
				echo -e ">  Please enter the starting line number from which to continue the blinds operation: (use 'exit' to stop the script)"
				#read linenum
				#linenum_check=$(echo $linenum |grep -e "^[0-9]*$")
				linenum_check=""
				while [ -z $linenum_check ];do
					read linenum
					killit=$(echo $linenum |grep -ie "exit")
					if [ $killit ];then exit;fi;
					linenum_check=$(echo $linenum |grep -e "^[0-9]*$")
					if [ $linenum_check ];then
						if [ $linenum -gt $nrows_small ];then
							linenum_check=""
							echo -e ">> Error: Number is larger than the length of the smallest available results file: $nrows_small rows"
							echo -e ">  Please enter a valid starting line number: (use 'exit' to stop the script)"
						fi
					else
						echo -e ">> Error: Input must be a positive integer value."
						echo -e ">  Please enter a valid starting line number: (use 'exit' to stop the script)"
					fi
				done
				
				count_line=$(($linenum+0))
				
				#count_line=$(($nrows_small+1))
				#echo ">> Continuing blinds operation from row/line $count_line..."
				#echo "[$(date +%Y-%m-%d-%H%M%S)]  Continuing blinds operation from row/line $count_line..." >> $log
			else
				if [ ! $nrows -eq $nrows_o ];then
					echo -e ">> Error: The following files are of different length (headers not counted):\n\t${outmerge} ($nrows lines)\n\t${outmerge_o} ($nrows_o lines)"
					echo -e "[$(date +%Y-%m-%d-%H%M%S)] Error: The following files are of different length:\n\t${outmerge}\n\t${outmerge_o}" >> $log
				fi
				if [ ! $nrows -eq $nrows_c ];then
					echo -e ">> Error: The following files are of different length (headers not counted):\n\t${outmerge} ($nrows lines)\n\t${outmerge_c} ($nrows_c lines)"
					echo -e "[$(date +%Y-%m-%d-%H%M%S)] Error: The following files are of different length:\n\t${outmerge}\n\t${outmerge_c}" >> $log
				fi
				if [ ! $nrows -eq $nrows_sd ];then
					echo -e ">> Error: The following files are of different length (headers not counted):\n\t${outmerge} ($nrows lines)\n\t${outmerge_sd} ($nrows_sd lines)"
					echo -e "[$(date +%Y-%m-%d-%H%M%S)] Error: The following files are of different length:\n\t${outmerge}\n\t${outmerge_sd}" >> $log
				fi
				if [ ! $nrows -eq $nrows_sch ];then
					echo -e ">> Error: The following files are of different length (headers not counted):\n\t${outmerge} ($nrows lines)\n\t${outsched} ($nrows_sch lines)"
					echo -e "[$(date +%Y-%m-%d-%H%M%S)] Error: The following files are of different length:\n\t${outmerge}\n\t${outmerge_sd}" >> $log
				fi
				
				#echo -e "> If you are trying to continue a previous run, make sure the following results files are of the same length and re-run this script:\n\t${outmerge}\n\t${outmerge_o}\n\t${outmerge_c}\n\t${outmerge_sd}\n\t${outsched}"
				#echo -e "> Altenatively, you may enter the starting line number from which to continue the blinds operation:"
				echo -e ">  Please enter the starting line number from which to continue the blinds operation: (use 'exit' to stop the script)"
				
				linenum_check=""
				while [ -z $linenum_check ];do
					read linenum
					killit=$(echo $linenum |grep -ie "exit")
					if [ $killit ];then exit;fi;
					linenum_check=$(echo $linenum |grep -e "^[0-9]*$")
					if [ $linenum_check ];then
						if [ $linenum -gt $nrows_small ];then
							linenum_check=""
							echo -e ">> Error: Number is larger than the length of the smallest available results file: $nrows_small rows"
							echo -e ">  Please enter a valid starting line number: (use 'exit' to stop the script)"
						fi
					else
						echo -e ">> Error: Input must be a positive integer value."
						echo -e ">  Please enter a valid starting line number: (use 'exit' to stop the script)"
					fi
				done
				
				count_line=$(($linenum+0))
				
				#linenum_check=""
				#while [ -z $linenum_check ] && [ $linenum -gt $nrows_small ];do
				#	echo -e ">> Error: Number is larger than the length of the smallest available results file: $nrows_small rows"
				#	echo -e "> Please enter a valid starting row/line number:"
				#	read linenum
				#	linenum_check=$(echo $linenum |grep -e "^[0-9]*$")
				#done
				
				#count_line=$(($linenum+0))
			fi
			
			#count_line=$(($nrows+1))
			echo ">  Continuing blinds operation from line $count_line..."
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Continuing blinds operation from line $count_line..." >> $log
			
			tail -n+$((${count_line}+1)) $timestep > $tmptimestep
			#head -$((${count_line}+2)) $outmerge > $tmpdir/tmp_outmerge.ill
			#head -$((${count_line}+2)) $outmerge_o > $tmpdir/tmp_outmerge_o.ill
			#head -$((${count_line}+2)) $outmerge_c > $tmpdir/tmp_outmerge_c.ill
			#head -$((${count_line}+2)) $outmerge_sd > $tmpdir/tmp_outmerge_sd.ill
			#head -$((${count_line}+1)) $outsched > $tmpdir/tmp_outsched.ill
			
			mkdir -p $resdir/backup
			
			if [ -f $resdir/backup/$(basename ${outmerge}) ];then
				echo -e ">> Warning: Backup already exists for $(basename ${outmerge})"
				echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup already exists for $(basename ${outmerge})" >> $log
				echo -e ">  Would you like to overwrite it? (Y/N)"
				read confirm
				confirm_check=$(echo $confirm |grep -ie "^Y$")
				if [ $confirm_check ];then
					cat $outmerge > $resdir/backup/$(basename ${outmerge}) #$(basename ${outmerge} .${outmerge##*.})
					#cat $outmerge_rflux > $resdir/backup/$(basename ${outmerge_rflux}) #$(basename ${outmerge} .${outmerge##*.})
					echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup overwritten: $resdir/backup/$(basename ${outmerge})" >> $log
					#head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_rflux}) > $outmerge_rflux
				fi
				head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge}) > $outmerge
			else
				cat $outmerge > $resdir/backup/$(basename ${outmerge}) #$(basename ${outmerge} .${outmerge##*.})
				#cat $outmerge_rflux > $resdir/backup/$(basename ${outmerge_rflux}) #$(basename ${outmerge} .${outmerge##*.})
				echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup saved: $resdir/backup/$(basename ${outmerge})" >> $log
				head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge}) > $outmerge
			fi
			if [ -f $resdir/backup/$(basename ${outmerge_o}) ];then
				echo -e ">> Warning: Backup already exists for $(basename ${outmerge_o})"
				echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup already exists for $(basename ${outmerge_o})" >> $log
				echo -e ">  Would you like to overwrite it? (Y/N)"
				read confirm
				confirm_check=$(echo $confirm |grep -ie "^Y$")
				if [ $confirm_check ];then
					cat $outmerge_o > $resdir/backup/$(basename ${outmerge_o})
					#cat $outmerge_o_rflux > $resdir/backup/$(basename ${outmerge_o_rflux})
					echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup overwritten: $resdir/backup/$(basename ${outmerge_o})" >> $log
					#head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_o_rflux}) > $outmerge_o_rflux
				fi
				head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_o}) > $outmerge_o
			else
				cat $outmerge_o > $resdir/backup/$(basename ${outmerge_o})
				#cat $outmerge_o_rflux > $resdir/backup/$(basename ${outmerge_o_rflux})
				echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup saved: $resdir/backup/$(basename ${outmerge_o})" >> $log
				head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_o}) > $outmerge_o
			fi
			if [ -f $resdir/backup/$(basename ${outmerge_c}) ];then
				echo -e ">> Warning: Backup already exists for $(basename ${outmerge_c})"
				echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup already exists for $(basename ${outmerge_c})" >> $log
				echo -e ">  Would you like to overwrite it? (Y/N)"
				read confirm
				confirm_check=$(echo $confirm |grep -ie "^Y$")
				if [ $confirm_check ];then
					cat $outmerge_c > $resdir/backup/$(basename ${outmerge_c})
					#cat $outmerge_c_rflux > $resdir/backup/$(basename ${outmerge_c_rflux})
					echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup overwritten: $resdir/backup/$(basename ${outmerge_c})" >> $log
					#head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_c_rflux}) > $outmerge_c_rflux
				fi
				head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_c}) > $outmerge_c
			else
				cat $outmerge_c > $resdir/backup/$(basename ${outmerge_c})
				#cat $outmerge_c_rflux > $resdir/backup/$(basename ${outmerge_c_rflux})
				echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup saved: $resdir/backup/$(basename ${outmerge_c})" >> $log
				head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_c}) > $outmerge_c
			fi
			if [ -f $resdir/backup/$(basename ${outmerge_sd}) ];then
				echo -e ">> Warning: Backup already exists for $(basename ${outmerge_sd})"
				echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup already exists for $(basename ${outmerge_sd})" >> $log
				echo -e ">  Would you like to overwrite it? (Y/N)"
				read confirm
				confirm_check=$(echo $confirm |grep -ie "^Y$")
				if [ $confirm_check ];then
					cat $outmerge_sd > $resdir/backup/$(basename ${outmerge_sd})
					echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup overwritten: $resdir/backup/$(basename ${outmerge_sd})" >> $log
				fi
				head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_sd}) > $outmerge_sd
			else
				cat $outmerge_sd > $resdir/backup/$(basename ${outmerge_sd})
				echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup saved: $resdir/backup/$(basename ${outmerge_sd})" >> $log
				head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_sd}) > $outmerge_sd
			fi
			if [ -f $resdir/backup/$(basename ${outsched}) ];then
				echo -e ">> Warning: Backup already exists for $(basename ${outsched})"
				echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup already exists for $(basename ${outsched})" >> $log
				echo -e ">  Would you like to overwrite it? (Y/N)"
				read confirm
				confirm_check=$(echo $confirm |grep -ie "^Y$")
				if [ $confirm_check ];then
					cat $outsched > $resdir/backup/$(basename ${outsched})
					echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup overwritten: $resdir/backup/$(basename ${outsched})" >> $log
				fi
				head -$((${count_line}+1)) $resdir/backup/$(basename ${outsched}) > $outsched
			else
				cat $outsched > $resdir/backup/$(basename ${outsched})
				echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Backup saved: $resdir/backup/$(basename ${outsched})" >> $log
				head -$((${count_line}+1)) $resdir/backup/$(basename ${outsched}) > $outsched
			fi
			
			#head -$((${count_line}+2)) $outmerge > $tmpdir/tmp_outmerge.ill
			#head -$((${count_line}+2)) $outmerge_o > $tmpdir/tmp_outmerge_o.ill
			#head -$((${count_line}+2)) $outmerge_c > $tmpdir/tmp_outmerge_c.ill
			#head -$((${count_line}+2)) $outmerge_sd > $tmpdir/tmp_outmerge_sd.ill
			#head -$((${count_line}+1)) $outsched > $tmpdir/tmp_outsched.ill
			
			#cat $tmpdir/tmp_outmerge.ill > $outmerge
			#cat $tmpdir/tmp_outmerge_o.ill > $outmerge_o
			#cat $tmpdir/tmp_outmerge_c.ill > $outmerge_c
			#cat $tmpdir/tmp_outmerge_sd.ill > $outmerge_sd
			#cat $tmpdir/tmp_outsched.ill > $outsched
			
			#head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge}) > $outmerge
			#head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_rflux}) > $outmerge_rflux
			#head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_o}) > $outmerge_o
			#head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_o_rflux}) > $outmerge_o_rflux
			#head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_c}) > $outmerge_c
			#head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_c_rflux}) > $outmerge_c_rflux
			#head -$((${count_line}+2)) $resdir/backup/$(basename ${outmerge_sd}) > $outmerge_sd
			#head -$((${count_line}+1)) $resdir/backup/$(basename ${outsched}) > $outsched
			
			echo -e ">  Continuing blinds operation..."
			echo -e "[$(date +%Y-%m-%d-%H%M%S)]  Continuing blinds operation..." >> $log
		else
			#count_line=1
			
			cat $tmpgrid > $outmerge
			#cat $tmpgrid > $outmerge_rflux
			echo -n "daylight_autonomy " > $outda
			#echo -n "daylight_autonomy " > $outda_rflux
			echo -n "daylight_autonomy_open " > $outda_o
			#echo -n "daylight_autonomy_open " > $outda_o_rflux
			echo -n "daylight_autonomy_closed " > $outda_c
			#echo -n "daylight_autonomy_closed " > $outda_c_rflux
			echo -n "" > $outase_hours
			cat $tmpgrid > $outmerge_o
			#cat $tmpgrid > $outmerge_o_rflux
			cat $tmpgrid > $outmerge_c
			#cat $tmpgrid > $outmerge_c_rflux
			cat $tmpgrid > $outmerge_sd
			echo "month day time $(echo $winlist |tr '\n' ' ' |sed -e "s/.rad//g;s/ $//")" > $outsched
			
			cat $timestep > $tmptimestep
			#echo "New... $count_line"
			
			echo ">  Starting blinds operation..."
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Starting blinds operation..." >> $log
		fi
		
		count_line=1
		
		#outmerge="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--final.ill"
		#cat $tmpgrid > $outmerge
        #echo -n "" > $outmerge
        #outda="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--da.ill"
		#cat $tmpgrid > $outmerge |sed -e "s/^[ ]\+/ /" > $outda
        #echo -n "daylight_autonomy " > $outda
        #outda_o="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--da_open.ill"
        #echo -n "daylight_autonomy_open " > $outda_o
        #outda_c="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--da_closed.ill"
        #echo -n "daylight_autonomy_closed " > $outda_c
        #outase_hours="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--ase_hours.ill"
        #echo -n "" > $outase_hours
        #outmerge_o="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--${bsdf_open}.ill"
		#cat $tmpgrid > $outmerge_o
        #echo -n "" > $outmerge_o
        #outmerge_c="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--${bsdf_closed}.ill"
        #cat $tmpgrid > $outmerge_c
		#echo -n "" > $outmerge_c
        #outmerge_sd="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--solardisc.ill"
        #cat $tmpgrid > $outmerge_sd
		#echo -n "" > $outmerge_sd
        #outsched="$resdir/${ptsname}--${weaname}--m${granularity}${rztag}--final_blinds_sched.dat"
        #echo "month day time $(echo $winlist |tr '\n' ' ' |sed -e "s/.rad//g;s/ $//")" > $outsched
		
        #count_line=1
		while read line; do
			hour=$(echo $line |cut -d "$delimiter" -f3)
			master_timestamp=$(echo $line |cut -d "$delimiter" -f1-3)
			echo $master_timestamp > $tmpstep
			blindsched="${master_timestamp} "
			echo -n "" > $tmpmerge
			echo -n "" > $tmpmerge_o
			echo -n "" > $tmpmerge_c
			#echo -n "" > $tmpmerge_rflux
			#echo -n "" > $tmpmerge_o_rflux
			#echo -n "" > $tmpmerge_c_rflux
			echo -n "" > $tmpmerge_sd
			#check_preiodstart=$(bc <<< "${hour} >= ${period_start}")
			check_periodstart=$(echo "${hour} ${period_start}" |awk '{if ($1 >= $2) print 1; else print 0}')
			#check_periodend=$(bc <<< "${hour} <= ${period_end}")
			check_periodend=$(echo "${hour} ${period_end}" |awk '{if ($1 <= $2) print 1; else print 0}')
			
			if [ $check_periodstart -eq 1 ] && [ $check_periodend -eq 1 ];then
				# Output the percent of the floor area that is in direct sunlight for each window group..
				# Find columnnumber based on timestamp.. $(cat $timestep |grep -n -e "^${master_timestamp}$" |cut -d ":" -f1)
				echo -n "" > $tmp2per
				for w in $winlist; do
					winname=$(basename $w .${w##*.})
					cat "$resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.dat" |cut -d"${delimiter}" -f${count_line} |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//" |awk -F"${delimiter}" -v trigger=$sd_trigger -v name=$winname 'BEGIN{count=0}{split($0,a);for(i=1;i<=NF;i++){if(a[i]>=trigger){count++}}}END{printf(name" %.2f\n",count/NF*100)}' >> $tmp2per
				done
				
				# Evaluate..
				#cat $tmp2per |sort -rn -k 2 > $tmp2per_sort
				sort -rn -k 2 $tmp2per > $tmp2per_sort
				trig_2per=$(cat "$tmp2per" |cut -d ' ' -f2 |total |awk '{printf("%.2f",$1)}')
				sum_2per=0
				off_ttl=0
				off=""
				#check_trigger=$(bc <<< "${trig_2per} >= ${sd_trigger_per}")
				check_trigger=$(echo "${trig_2per} ${sd_trigger_per}" |awk '{if ($1 >= $2) print 1; else print 0}')
				if [ $check_trigger -eq 1 ];then
					echo -e "$(echo $master_timestamp |tr ' ' '\t')\t  | $trig_2per% of sensors are above $sd_trigger lux, evaluating window groups." >> $log
					while read line_2per; do
						name_2per=$(echo $line_2per |cut -d ' ' -f1)
						value_2per=$(echo $line_2per |cut -d ' ' -f2)
						sum_2per=$(echo ${sum_2per} ${value_2per} |awk '{printf("%.2f",$1+$2)}')
						sum_check=$(echo ${trig_2per} ${sum_2per} |awk '{printf("%.2f",$1-$2)}')
						echo -e "\t\t\t  |   >> $name_2per is contributing $value_2per% of the total $trig_2per% in the space, closing." >> $log
						off="${off} ${name_2per}"
						off_ttl=$(echo ${off_ttl} ${value_2per} |awk '{printf("%.2f",$1+$2)}')
						#check_trigger=$(bc <<< "${sum_check} < ${sd_trigger_per}")
						check_trigger=$(echo "${sum_check} ${sd_trigger_per}" |awk '{if ($1 < $2) print 1; else print 0}')
						if [ $check_trigger -eq 1 ];then
							echo -e "\t\t\t  |   >> Less than $sd_trigger_per% achieved by closing: $off (-$off_ttl%)" >> $log
							break
						fi
					done < "$tmp2per_sort"
				else
					echo -e "$(echo $master_timestamp |tr ' ' '\t')\t  | Less than $sd_trigger_per% of the analysis area is under direct sunlight: ${trig_2per}%" >> $log
				fi
			else
				off=$winlist
				echo -e "$(echo $master_timestamp |tr ' ' '\t')\t  |   >> Out of operable range, closing all." >> $log
			fi
			
			# Outputs..
			for w in $winlist; do
				winname=$(basename $w .${w##*.})
				check_winname=$(echo $off |grep -o -e "$winname")
				if [ $check_winname ];then
					cut -d "${delimiter}" -f${count_line} "$resdir_dat/${ptsname}--${winname}--${bsdf_closed}--${weaname}--m${granularity}${rztag}.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/${winname} /" >> $tmpmerge
					#cut -d "${delimiter}" -f${count_line} "$resdir_dat/${ptsname}--${winname}--${bsdf_closed}--${weaname}--m${granularity}${rztag}_rflux.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/${winname} /" >> $tmpmerge_rflux
					blindsched="${blindsched}c "
				else
					cut -d "${delimiter}" -f${count_line} "$resdir_dat/${ptsname}--${winname}--${bsdf_open}--${weaname}--m${granularity}${rztag}.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/${winname} /" >> $tmpmerge
					#cut -d "${delimiter}" -f${count_line} "$resdir_dat/${ptsname}--${winname}--${bsdf_open}--${weaname}--m${granularity}${rztag}_rflux.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/${winname} /" >> $tmpmerge_rflux
					blindsched="${blindsched}o "
				fi
				echo >> $tmpmerge
				cut -d "${delimiter}" -f${count_line} "$resdir_dat/${ptsname}--${winname}--${bsdf_open}--${weaname}--m${granularity}${rztag}.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/${winname} /" >> $tmpmerge_o
				echo >> $tmpmerge_o
				cut -d "${delimiter}" -f${count_line} "$resdir_dat/${ptsname}--${winname}--${bsdf_closed}--${weaname}--m${granularity}${rztag}.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/${winname} /" >> $tmpmerge_c
				echo >> $tmpmerge_c
				
				#echo >> $tmpmerge_rflux
				#cut -d "${delimiter}" -f${count_line} "$resdir_dat/${ptsname}--${winname}--${bsdf_open}--${weaname}--m${granularity}${rztag}_rflux.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/${winname} /" >> $tmpmerge_o_rflux
				#echo >> $tmpmerge_o_rflux
				#cut -d "${delimiter}" -f${count_line} "$resdir_dat/${ptsname}--${winname}--${bsdf_closed}--${weaname}--m${granularity}${rztag}_rflux.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/${winname} /" >> $tmpmerge_c_rflux
				#echo >> $tmpmerge_c_rflux
				
				cut -d "${delimiter}" -f${count_line} "$resdir_sd/${ptsname}--${winname}--${weaname}--m${granularity}${rztag}.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/${winname} /" >> $tmpmerge_sd
				echo >> $tmpmerge_sd
			done
			
			# Add non-BSDF contribution..
			if [ $add ];then
				cut -d "${delimiter}" -f${count_line} "$resdir_add/${ptsname}--${weaname}--m${granularity}${rztag}.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/nonbsdf /" >> $tmpmerge
				echo >> $tmpmerge
				cut -d "${delimiter}" -f${count_line} "$resdir_add/${ptsname}--${weaname}--m${granularity}${rztag}.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/nonbsdf /" >> $tmpmerge_o
				echo >> $tmpmerge_o
				cut -d "${delimiter}" -f${count_line} "$resdir_add/${ptsname}--${weaname}--m${granularity}${rztag}.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/nonbsdf /" >> $tmpmerge_c
				echo >> $tmpmerge_c
				
				#cut -d "${delimiter}" -f${count_line} "$resdir_add/${ptsname}--${weaname}--m${granularity}${rztag}_rflux.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/nonbsdf /" >> $tmpmerge_rflux
				#echo >> $tmpmerge_rflux
				#cut -d "${delimiter}" -f${count_line} "$resdir_add/${ptsname}--${weaname}--m${granularity}${rztag}_rflux.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/nonbsdf /" >> $tmpmerge_o_rflux
				#echo >> $tmpmerge_o_rflux
				#cut -d "${delimiter}" -f${count_line} "$resdir_add/${ptsname}--${weaname}--m${granularity}${rztag}_rflux.dat" |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/nonbsdf /" >> $tmpmerge_c_rflux
				#echo >> $tmpmerge_c_rflux
				
				cut -d "${delimiter}" -f${count_line} "$resdir_sd/${ptsname}--nonoperated--${weaname}--m${granularity}${rztag}.dat"  |tr '\n' "${delimiter}" |sed -e "s/${delimiter}$//;s/ \+/ /g;s/^/nonbsdf /" >> $tmpmerge_sd
				echo >> $tmpmerge_sd
			fi
			
			echo $blindsched >> $outsched
			cat $tmpmerge |cut -d "${delimiter}" -f2- |total -t" " |paste -d "${delimiter}" $tmpstep - |sed -e "s/ $//" >> $outmerge
			cat $tmpmerge_o |cut -d "${delimiter}" -f2- |total -t" " |paste -d "${delimiter}" $tmpstep - |sed -e "s/ $//" >> $outmerge_o
			cat $tmpmerge_c |cut -d "${delimiter}" -f2- |total -t" " |paste -d "${delimiter}" $tmpstep - |sed -e "s/ $//" >> $outmerge_c
			
			#cat $tmpmerge_rflux |cut -d "${delimiter}" -f2- |total -t" " |paste -d "${delimiter}" $tmpstep - |sed -e "s/ $//" >> $outmerge_rflux
			#cat $tmpmerge_o_rflux |cut -d "${delimiter}" -f2- |total -t" " |paste -d "${delimiter}" $tmpstep - |sed -e "s/ $//" >> $outmerge_o_rflux
			#cat $tmpmerge_c_rflux |cut -d "${delimiter}" -f2- |total -t" " |paste -d "${delimiter}" $tmpstep - |sed -e "s/ $//" >> $outmerge_c_rflux
			
			cat $tmpmerge_sd |cut -d "${delimiter}" -f2- |total -t" " |paste -d "${delimiter}" $tmpstep - |sed -e "s/ $//" >> $outmerge_sd
			
			#((count_line++))
			count_line=$((count_line+1))
			
        done < $tmptimestep
        echo ""  >> $log
		
        # Calculate sDA
        echo "[$(date +%Y-%m-%d-%H%M%S)]  Calculating Daylight Autonomy with a threshold of ${da_trigger} lux (DA$da_trigger): $outda" >> $log
        ncols=$(($(cat $tmpdir/$pts |sed -e "s/^$//g" |wc -l)+3))
        for col in $(seq 4 $ncols); do
            cat $outmerge |grep -A $period_int -e "^[0-9]\{1,2\} [0-9]\{1,2\} $period_start " |sed -e "/--/d" |cut -d ' ' -f$col |awk -v trigger=$da_trigger 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("%.2f ",count/num*100)}' >> $outda
            cat $outmerge_o |grep -A $period_int -e "^[0-9]\{1,2\} [0-9]\{1,2\} $period_start " |sed -e "/--/d" |cut -d ' ' -f$col |awk -v trigger=$da_trigger 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("%.2f ",count/num*100)}' >> $outda_o
            cat $outmerge_c |grep -A $period_int -e "^[0-9]\{1,2\} [0-9]\{1,2\} $period_start " |sed -e "/--/d" |cut -d ' ' -f$col |awk -v trigger=$da_trigger 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("%.2f ",count/num*100)}' >> $outda_c
			#cat $outmerge_rflux |grep -A $period_int -e "^[0-9]\{1,2\} [0-9]\{1,2\} $period_start " |sed -e "/--/d" |cut -d ' ' -f$col |awk -v trigger=$da_trigger 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("%.2f ",count/num*100)}' >> $outda_rflux
            #cat $outmerge_o_rflux |grep -A $period_int -e "^[0-9]\{1,2\} [0-9]\{1,2\} $period_start " |sed -e "/--/d" |cut -d ' ' -f$col |awk -v trigger=$da_trigger 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("%.2f ",count/num*100)}' >> $outda_o_rflux
            #cat $outmerge_c_rflux |grep -A $period_int -e "^[0-9]\{1,2\} [0-9]\{1,2\} $period_start " |sed -e "/--/d" |cut -d ' ' -f$col |awk -v trigger=$da_trigger 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("%.2f ",count/num*100)}' >> $outda_c_rflux
		done
        echo "[$(date +%Y-%m-%d-%H%M%S)]  Calculating Spatial Daylight Autonomy (DA$da_trigger/$sda_trigger%): $outlm83" >> $log
        echo -e "Calculating LM83 metrics for the \"$ptsname\" analysis grid and \"$weaname\" weather file.." >> $outlm83
        echo -e "\tSpatial Daylight Autonomy:" >> $outlm83
        cat $outda |tr ' ' '\n' |awk -v trigger=$sda_trigger 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("\t\t%.2f%s\toperated\n",count/num*100,"%")}' >> $outlm83
        cat $outda_o |tr ' ' '\n' |awk -v trigger=$sda_trigger -v name=${bsdf_open} 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("\t\t%.2f%s\t"name"\n",count/num*100,"%")}' >> $outlm83
        cat $outda_c |tr ' ' '\n' |awk -v trigger=$sda_trigger -v name=${bsdf_closed} 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("\t\t%.2f%s\t"name"\n",count/num*100,"%")}' >> $outlm83
        #cat $outda_rflux |tr ' ' '\n' |awk -v trigger=$sda_trigger 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("\t\t%.2f%s\toperated\n",count/num*100,"%")}' >> $outlm83
        #cat $outda_o_rflux |tr ' ' '\n' |awk -v trigger=$sda_trigger -v name=${bsdf_open} 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("\t\t%.2f%s\t"name"\n",count/num*100,"%")}' >> $outlm83
        #cat $outda_c_rflux |tr ' ' '\n' |awk -v trigger=$sda_trigger -v name=${bsdf_closed} 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("\t\t%.2f%s\t"name"\n",count/num*100,"%")}' >> $outlm83
        
        # Calculate ASE
        echo "[$(date +%Y-%m-%d-%H%M%S)]  Evaluating hours of direct exposure with a threshold of ${ase_trigger_lux} lux: $outase_hours" >> $log
        echo -n -e "\tAnnual Sunlight Exposure: " >> $outlm83
        ttl_hours=$(cat $outmerge_sd |grep -A $period_int -e "^[0-9]\{1,2\} [0-9]\{1,2\} $period_start " |sed -e "/--/d" |wc -l)
        ncols=$(($(cat $tmpdir/$pts |sed -e "/^$/d" |wc -l)+3))
        for col in $(seq 4 $ncols); do
            cat $outmerge_sd |grep -A $period_int -e "^[0-9]\{1,2\} [0-9]\{1,2\} $period_start " |sed -e "/--/d" |cut -d ' ' -f$col |awk -v trigger=$ase_trigger_lux 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("%.0f ",count)}' >> $outase_hours
        done
        echo "[$(date +%Y-%m-%d-%H%M%S)]  Calculating Annual Sunlight Exposure (ASE$ase_trigger_lux,$ase_trigger_hours): $outase" >> $log
        cat $outase_hours |tr ' ' '\n' |awk -v trigger=$ase_trigger_hours 'BEGIN{count=0;num=0}{if($1>=trigger){count++};num++}END{printf("%.2f%s\n\n",count/num*100,"%")}' >> $outlm83

    done
done
STOPTWO
rmlist=$(ls $weadir |grep -v -i -e ".epw$")
for f in $rmlist; do rm -f -r $weadir/$f;done;

echo ""  >> $log
echo -e ">  The results can be found in the \"$resdir/\" directory under the following naming convention:\n   [pointfile]--[weatherfile]--m[N]--final.ill" >> $log
echo -e ">  The blinds schedule can be found in the \"$resdir/\" directory under the following naming convention:\n   [pointfile]--[weatherfile]--m[N]--final_blinds_sched.dat" >> $log
echo -e ">  The Daylight Autonomy (DA$da_trigger) calculation can be found in the \"$resdir/\" directory under the following naming convention:\n   [pointfile]--[weatherfile]--m[N]--da.ill" >> $log
echo -e ">  The Spatial Daylight Autonomy (DA$da_trigger/$sda_trigger) calculation(s) can be found in the \"$resdir/\" directory: $outlm83" >> $log
echo ""  >> $log

timeend=$(date +%Y-%m-%d-%H%M%S)

# Calculate time elapsed..
start=$(echo $timestart |grep -e "[0-9]" |cut -d '-' -f4)
end=$(echo $timeend |grep -e "[0-9]" |cut -d '-' -f4)
start_day=$(echo $timestart |grep -e "[0-9]" |cut -d '-' -f3)
end_day=$(echo $timeend |grep -e "[0-9]" |cut -d '-' -f3)
start_mo=$(echo $timestart |grep -e "[0-9]" |cut -d '-' -f2)
end_mo=$(echo $timeend |grep -e "[0-9]" |cut -d '-' -f2)
start_yr=$(echo $timestart |grep -e "[0-9]" |cut -d '-' -f1)
end_yr=$(echo $timeend |grep -e "[0-9]" |cut -d '-' -f1)
years=$(echo $end_yr $start_yr |awk '{printf "%.4d", $1-$2}')
months=$(echo $end_mo $start_mo |awk '{printf "%.2d", $1-$2}')
days=$(echo $end_day $start_day |awk '{printf "%.2d", $1-$2}')
#check_time_first=$(bc <<< "${end:0:2} == 24")
#check_time_first=$(echo "${end:0:2} 24" |awk '{if ($1 == $2) print 1; else print 0}')
checktime_e=$(echo $end |cut -c 1-2)
check_time_first=$(echo "$checktime_e 24" |awk '{if ($1 == $2) print 1; else print 0}')
#check_time_second=$(bc <<< "${start:0:2} == 24")
#check_time_second=$(echo "${start:0:2} 24" |awk '{if ($1 == $2) print 1; else print 0}')
checktime_s=$(echo $start |cut -c 1-2)
check_time_second=$(echo "$checktime_s 24" |awk '{if ($1 == $2) print 1; else print 0}')

if [ $check_time_first -eq 1 ];then
    hours=$(echo 00 $checktime_s |awk '{printf "%.2d", $1-$2}')
	#echo 00 $checktime_s |awk '{printf "%.2d", $1-$2}'
    days=$(echo $days |awk '{printf "%.2d", $1+1}')
elif [ $check_time_second -eq 1 ];then
    hours=$(echo $checktime_e 00 |awk '{printf "%.2d", $1-$2}')
	#echo $checktime_e 00 |awk '{printf "%.2d", $1-$2}'
    days=$(echo $days |awk '{printf "%.2d", $1+1}')
else
    hours=$(echo $checktime_e $checktime_s |awk '{printf "%.2d", $1-$2}')
fi

checktime_e=$(echo $end |cut -c 3-4)
checktime_s=$(echo $start |cut -c 3-4)
minutes=$(echo $checktime_e $checktime_s |awk '{printf "%.2d", $1-$2}')
checktime_e=$(echo $end |cut -c 5-6)
checktime_s=$(echo $start |cut -c 5-6)
seconds=$(echo $checktime_e $checktime_s |awk '{printf "%.2d", $1-$2}')

sec="";min="";hr="";day="";
#check_first=$(bc <<< "(${hours}*60+$minutes) >= 60")
check_first=$(echo "(${hours}*60+$minutes) 60" |awk '{if ($1 >= $2) print 1; else print 0}')
#check_second=$(bc <<< "${days} > 0")
check_second=$(echo "${days} 0" |awk '{if ($1 > $2) print 1; else print 0}')
#check_third=$(bc <<< "(${minutes#-?}*60+${seconds#-?}) >= 60")
check_third=$(echo "(${minutes#-?}*60+${seconds#-?}) 60" |awk '{if ($1 >= $2) print 1; else print 0}')

if [ $check_first -eq 1 ] || [ $check_second -eq 1 ];then hr=1;elif [ $check_third -eq 1 ];then min=1;else sec=1;fi;
#echo $days $hours $minutes $seconds
if [ $sec ];then elapsed=$(echo $days $hours $minutes $seconds |awk '{printf ($1*24*60*60+$2*60*60+$3*60+$4);printf " second(s).\n"}');fi;
if [ $min ];then elapsed=$(echo $days $hours $minutes $seconds |awk '{min=$1*24*60+$2*60+$3;sec=$4*1;}{if(sec < 0){min=min-1;sec=60+sec}}{printf min;printf " minutes(s) and ";printf "%.0f", sec;printf " second(s).\n"}');fi;
if [ $hr ];then elapsed=$(echo $days $hours $minutes $seconds |awk '{hr=$1*24+$2;min=$3*1;sec=$4*1;}{if(sec < 0){min=min-1;sec=60+sec}}{if(min < 0){hr=hr-1;min=60+min}}{printf hr;printf " hours(s), ";printf min;printf " minutes(s) and ";printf "%.0f", sec;printf " second(s).\n"}');fi;
if [ $day ];then elapsed=$(echo $days $hours $minutes $seconds |awk '{day=$1*1;hr=$2*1;min=$3*1;sec=$4*1;}{if(sec < 0){min=min-1;sec=60+sec}}{if(min < 0){hr=hr-1;min=60+min}}{if(hr < 0){day=day-1;hr=24+hr}}{printf day;printf " day(s), ";printf hr;printf " hours(s), ";printf min;printf " minutes(s) and ";printf "%.0f", sec;printf " second(s).\n"}');fi;

# Output time elapsed..
echo -e ">  Finished in $elapsed.\n   The results have been written to the ${resdir}/ directory and the log to the ${logdir}/ directory."
echo "[$timeend]  Finished in $elapsed" >> $log



#EOF
