#!/bin/bash

# Annual simulation using the rcontrib Radiance tool. This script will generate its own
# folder structure beyond the starting directories required, which are outlined below.
# 
# Version 1.0 (August 25, 2017)
# 
# Author: Alen Mahic, Ery Djunaedy (Energy Studies in Buildings Laboratory Universiy of Oregon; Integrated Design Lab University of Idaho)
# This work is licensed under the Creative Commons Attribution 3.0 Unported License. To view a copy of this license, visit http://creativecommons.org/licenses/by/3.0/.
# 
# Radiance 5.0+ is required. (NREL package)
# https://openstudio.nrel.gov/getting-started-developer/getting-started-radiance
# 
# Usage: ./rcontrib.sh -i [file] -m N -rz+/-N -clean
# 
#       -i [file]       The main .rad file for the model, which pulls its geometry elements from the ./objects/ directory
#                       and its windows from the ./objects/windows/ directory. Each window group is broken down into a
#                       .rad file of its own with the filename matching the modifier name  used for each defined polygon.
#                       These two must match. Required input.
# 
#       -m N            Specifies the number of subdivisions of the sky. The Reinhart MF: setting where "N" is 1, 2, 3,
#                       or 4. Optional input, the default value is 4.
# 
#       -rz+/-N         Specifies the orientation of the building in degrees. Make sure that there is no space between the
#                       "-rz" argument and the +/- "N" value. Optional input, the default value is 0.
# 
#       -clean          This switch will clean the directory and delete all directories and files that were generated in
#                       any previous runs. This includes the contents of the "results" folder.
# 
# 
# Assumed folder structure:
# 
#       ./
#           The main directory contains this script file and the primary .rad file of the model.
# 
#       ./data/
#           Contains the point analysis files.
# 
#       ./materials/
#           Contains the "materials.rad" file with the relevant material definitions for the model. The "glazing" and
#           "wall_mat" material identifiers are used in the script and assigned to the glazings and walls respectively.
#           These two must be defined in the "materials.rad" file.
# 
#       ./objects/
#           Contains the geometry .rad files referenced by the primary model file in the main directory.
# 
#       ./wea/
#           Contains the TMY3 weather files in .epw format.
# 
# Assumed input files:
# 
#       ./[filename].rad
#           Primary model file.
# 
#       ./data/*.pts
#           All desired analysis point files.
# 
#       ./materials/materials.rad
#           Materials file.
# 
#       ./objects/*.rad
#           All relevant geometry files referenced by the primary model file.
# 
#       ./wea/*.epw
#           All desired weather files.
# 
# Results:
#       The outputs will be deposited in the ./results/ directory. The naming conventions for the operated annual illuminance
#       matrix, the operation schedule files, and the LM-83 metric calculations are as follows:
#           [pointfile]--[weatherfile]--m[N].ill
# 
# 
# HDR dimensions...
dim="400"

# Read script arguments..
args=$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/-\{1,\} /-/g;s/ -/\t-/g")

# SOS..
help="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -x -e "-\{1,\}help" |tail -1 |sed -e "s/^-\{1,\}//")"
# Clean switch..
clean="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -x -e "-\{1,\}clean" |tail -1 |sed -e "s/^-\{1,\}//")"

# Directories
windir="./objects/windows"
ptsdir="./data"
if [ ! "$(ls $ptsdir)" ];then ">> Error: The \"$ptsdir\" is empty.";exit 0;fi;
matdir="./materials"
if [ ! "$(ls $matdir)" ];then ">> Error: The \"$matdir\" is empty.";exit 0;fi;
weadir="./wea"
if [ ! "$(ls $wadir)" ];then ">> Error: The \"$weadir\" is empty.";exit 0;fi;
vwdir="./views"
if [ $views ];then
	if [ ! "$(ls $wadir)" ];then ">> Error: The \"$weadir\" is empty.";exit 0;fi;
fi
resdir="./results"
mtxdir="${resdir}/mtx"
resviewdir="${resdir}/views"
optdir="./options"
skydir="./skies"
tmpdir="./temp"
logdir="./log"

# Temp and material outputs..
tmp_sky="temp_sky.rad"
tmp_wea="temp_weather.wea"
tmp_dat="temp.dat"
tmp_ill="temp.ill"

opt="$optdir/general.opt"
opt_view="$optdir/view.opt"
sky="./skies/sky_white_360.rad"
sky_blank="./skies/sky_blank.rad"

#scriptname=$(basename $0)

# Clean the directory..
if [ "$clean" ];then
    rm -f -r $tmpdir $resdir/* $dmx $vmx $opt $opt_view $sky $sky_blank
    rmlist=$(ls ./ |grep -i -e ".oct$")
    for f in $rmlist; do rm -f -r $f;done;
    rmlist=$(ls $weadir |grep -v -i -e ".epw$")
    for f in $rmlist; do rm -f -r $weadir/$f;done;
    echo ">  Directory cleaned."
    exit 0;
fi

# Parse model and parameter arguments..
model="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -e "-\{1,\}i " |tail -1 |sed "s/-\{1,\}i //;s/^\.\{1,\}//;s/^\/\{1,\}//;s/^/\.\//;s/\/$//")"
if [ ! "$model" ];then echo -e ">> Error: Please specify the input model.";exit 0;elif [ ! -f $model ];then echo ">> Error: The specified file does not exist.";exit 0;elif [ ! $(echo ".${model##*.}" |grep -e ".rad") ];then echo -e ">> Error: The specified file is not in .rad format.";exit 0;fi;
granularity="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -e "-\{1,\}m " |tail -1 |sed "s/-\{1,\}m //")"
if [ ! "$granularity" ];then granularity=4;fi;
rotation="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -e "-\{1,\}rz" |tail -1 |sed "s/-\{1,\}rz//")"
if [ ! "$rotation" ];then rotation=0;rztag="";else rztag="--rz$rotation";fi;
views="$(echo $@ |sed "s/^ *//;s/ *$//;s/ \{1,\}/ /g;s/\/\{1,\}/\//g;s/-\{1,\}/-/g;s/ -/#-/g" |tr '#' '\n' |grep -e "-\{1,\}view")"
views="" # Force annual HDR off for now..

# Model and materials..
modelname=$(basename $model .${model##*.})
materials="./materials/materials.rad"
if [ ! -f $materials ];then echo ">> Error: The \"$materials\" file does not exist.";exit 0;fi;

# Make relevant directories..
mkdir -p $resdir $mtxdir $matdir $optdir $skydir $tmpdir $logdir

if [ $views ];then mkdir -p $resviewdir;fi;

# Log
timestart=$(date +%Y-%m-%d-%H%M%S)
log="$logdir/log_${timestart}.log"
echo "[$timestart]  Starting.." > $log

dir="$( cd "$( dirname "$0" )" && pwd )"
echo -e ">  Running.\n>  Use the following command in a new terminal to follow the log file live:\n      tail -f ${dir}/log/$(ls ${dir}/log |tail -1)"

# Lists for loops..
ptslist=$(ls $ptsdir |grep -i -e ".pts$")
vwlist=$(ls $vwdir |grep -i -e ".vf$")
wealist=$(ls $weadir |grep -i -e ".epw$")

# Generate skies..
echo -e "void glow sky_glow 0 0 4 1 1 1 0\nsky_glow source sky 0 0 4 0 0 1 360" > $sky
echo -e "skyfunc glow sky_mat 0 0 4 1 1 1 0\nsky_mat source sky 0 0 4 0 0 1 180\nskyfunc glow ground_glow 0 0 4 1 .8 .5 0\nground_glow source ground 0 0 4 0 0 -1 180" > $sky_blank
echo "[$(date +%Y-%m-%d-%H%M%S)]  Sky files generated." >> $log

# Generate options files..
echo "-ab 6 -ad 4096 -as 1024 -ar 256 -aa 0.1 -lw 0.0003 -dt 0 -ds 0.1" > $opt
if [ $views ];then echo "-ab 10 -ad 10240 -as 5120 -ar 2560 -aa 0.08" > $opt_view;fi;
echo "[$(date +%Y-%m-%d-%H%M%S)]  Options files generated." >> $log

# Define the light material for the view matrix and define the materials required for both the daylight and view matrices.

oconv -w $materials $model $sky > ${modelname}_dmx.oct
echo "[$(date +%Y-%m-%d-%H%M%S)]  OCTREE generated: ./${modelname}_dmx.oct" >> $log

# Parse EPW weather file into WEA format for use with gendaymtx..
for wea in $wealist; do
    weafile="$weadir/$wea"
    weaname=$(basename $wea .${wea##*.})
    timestep="$weadir/${weaname}_timestep.dat"
    # Location is the first line of the EPW header..
    loc=$(head -1 $weafile)
    # Data period is the 8th line of the EPW header..
    dataper=$(head -8 $weafile |tail -1)
    lat=$(echo $loc |cut -d ',' -f7)
    longitude=$(echo $loc |cut -d ',' -f8)
    # EPW lists longitude + to the east, Radiance + to the west..
    lon=$(echo $longitude |rcalc -e '$1=-1*$1')
    meridian=$(echo $loc |cut -d ',' -f9)
    # EPW lists time zone in hours, but we need degrees..
    mer=$(echo $meridian |rcalc -e '$1=$1*-15')
    # WEA header format.. weather_data_file_units is either
    # field 2 (described as "#Number" in the EPW format documentation) or
    # field 3 (described as "#Number Records/Intervals in an hour" in the EPW format documentation).
    # [http://apps1.eere.energy.gov/buildings/energyplus/pdfs/weatherdatainformation.pdf]
    # This script uses field 2.
    echo -e "place $(echo $loc |cut -d "," -f2)_$(echo $loc |cut -d "," -f4)\nlatitude $lat\nlongitude $lon\ntime_zone $mer\nsite_elevation $(echo $loc |cut -d "," -f10)\nweather_data_file_units $(echo $dataper |cut -d "," -f2)" > "$weadir/${weaname}.wea"
    tail -n +9 $weafile |rcalc -t, -e '$1=$2;$2=$3;$3=$4;$4=$15;$5=$16' |sed -e "s/,/ /g" >> $weadir/$weaname.wea
    tail -n +7 $weadir/$weaname.wea |cut -d ' ' -f1-3 > $timestep
    # Generate sky vector matrix..
    if [ -f $weadir/${weaname}--m${granularity}${rztag}.smx ];then echo ">> Warning: Existing sky vector matrix found: $weadir/${weaname}--m${granularity}${rztag}.smx";
    else
        echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating sky vector matrix with an \"-m ${granularity}\" setting" >> $log
        gendaymtx -h -r $(awk -v rotation=$rotation 'BEGIN{print (rotation*-1);}') -m $granularity -of $weadir/$weaname.wea > "$weadir/${weaname}--m${granularity}${rztag}.smx"
        echo "[$(date +%Y-%m-%d-%H%M%S)]  Sky vector matrix generated: $weadir/${weaname}_m${granularity}${rztag}.smx" >> $log
    fi
done

for pts in $ptslist; do
	ptsname=$(basename ${pts} .${pts##*.})
	cat $ptsdir/$pts |sed -e "/^$\n/d" > $tmpdir/$pts
	#echo "" >> $tmpdir/$pts
	echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating daylight matrix for the \"${ptsname}\" analysis grid." >> $log
	rcontrib -fo -faf -I+ -e MF:${granularity} -f reinhart.cal -o $mtxdir/${modelname}_${ptsname}_m${granularity}.dmx -b rbin -bn Nrbins -m sky_glow @${opt} -w ${modelname}_dmx.oct < $tmpdir/$pts
	for wea in $wealist; do
		weaname=$(basename $wea .${wea##*.})
		outname="${ptsname}--${weaname}--m${granularity}${rztag}"
		timestep="$weadir/${weaname}_timestep.dat"
		echo -n "" > $tmpdir/$tmp_dat
		echo -n "" > $tmpdir/$tmp_ill
		echo "[$(date +%Y-%m-%d-%H%M%S)]  Running annual simulation for the \"${ptsname}\" analysis grid and the ${weaname} TMY3 weather file." >> $log
		dctimestep -n 8760 -if $mtxdir/${modelname}_${ptsname}_m${granularity}.dmx $weadir/${weaname}--m${granularity}${rztag}.smx |sed -e '/^$/d' |tail -n +9 |tr '\t' '#' >> $tmpdir/$tmp_dat
		while read line;do
			echo $line |awk -F"#" '{for(i=1;i<=NF;i++){split($i,a," ");conv=(179*(a[1]*0.265+a[2]*0.670+a[3]*0.065));if(conv==0){printf("%.0f ",conv)}else{printf("%.2f ",conv)}}}END{print ""}' |sed -e "s/ $//;s/ \+/#/g" >> $tmpdir/$tmp_ill
		done < $tmpdir/$tmp_dat
		ave_x=$(cat $tmpdir/$pts |cut -d' ' -f1 |tr ' ' '\n' |awk '{x+=$1;x_cnt+=1}END{printf("%.6f",x/x_cnt)}')
		max_x=$(cat $tmpdir/$pts |cut -d' ' -f1 |tr ' ' '\n' |awk 'NR==1{x_max=$1}{if($1>x_max)x_max=$1}END{printf("%.6f",x_max)}')
		ave_y=$(cat $tmpdir/$pts |cut -d' ' -f2 |tr ' ' '\n' |awk '{y+=$1;y_cnt+=1}END{printf("%.6f",y/y_cnt)}')
		max_y=$(cat $tmpdir/$pts |cut -d' ' -f2 |tr ' ' '\n' |awk 'NR==1{y_max=$1}{if($1>y_max)y_max=$1}END{printf("%.6f",y_max)}')
		ave_z=$(cat $tmpdir/$pts |cut -d' ' -f3 |tr ' ' '\n' |awk '{z+=$1;z_cnt+=1}END{printf("%.6f",z/z_cnt)}')
		max_z=$(cat $tmpdir/$pts |cut -d' ' -f3 |tr ' ' '\n' |awk 'NR==1{z_max=$1}{if($1>z_max)z_max=$1}END{printf("%.6f",z_max)}')
		#if [ echo "scale=6;1-(${ave_z}/${max_z}) > 0.01" |bc -l -eq 1 ];then

		if [ $(echo "scale=6;1-(${ave_y}/${max_y}) < 0.01" |bc -l) -eq 1 ] && [ $(echo "scale=6;1-(${ave_z}/${max_z}) > 0.01" |bc -l) -eq 1 ];then
			cat $tmpdir/$pts |cut -d' ' -f1 |tr "\\r\\n" " " |sed -e "s/[ ]\+/ /g;s/ $//;s/^/   /" > $resdir/${outname}.ill # |cut -d " " -f1 |tr "\\r\\n" " " |sed -e "s/ \+/ /g"
			echo "" >> $resdir/${outname}.ill
			cat $tmpdir/$pts |cut -d' ' -f3 |tr "\\r\\n" " " |sed -e "s/[ ]\+/ /g;s/ $//;s/^/   /" >> $resdir/${outname}.ill
			echo "" >> $resdir/${outname}.ill
			#echo "> X-axis surface normal.. ${ptsname} | x: $ave_x/$max_x | y: $ave_y/$max_y | z: $ave_z/$max_z"
		elif [ $(echo "scale=6;1-(${ave_x}/${max_x}) < 0.01" |bc -l) -eq 1 ] && [ $(echo "scale=6;1-(${ave_z}/${max_z}) > 0.01" |bc -l) -eq 1 ];then
			cat $tmpdir/$pts |cut -d' ' -f2 |tr "\\r\\n" " " |sed -e "s/[ ]\+/ /g;s/ $//;s/^/   /" > $resdir/${outname}.ill # |cut -d " " -f1 |tr "\\r\\n" " " |sed -e "s/ \+/ /g"
			echo "" >> $resdir/${outname}.ill
			cat $tmpdir/$pts |cut -d' ' -f3 |tr "\\r\\n" " " |sed -e "s/[ ]\+/ /g;s/ $//;s/^/   /" >> $resdir/${outname}.ill
			echo "" >> $resdir/${outname}.ill
			#echo "> Y-axis surface normal.. ${ptsname} | x: $ave_x/$max_x | y: $ave_y/$max_y | z: $ave_z/$max_z"
		else
			cat $tmpdir/$pts |cut -d' ' -f1 |tr "\\r\\n" " " |sed -e "s/[ ]\+/ /g;s/ $//;s/^/   /" > $resdir/${outname}.ill # |cut -d " " -f1 |tr "\\r\\n" " " |sed -e "s/ \+/ /g"
			echo "" >> $resdir/${outname}.ill
			cat $tmpdir/$pts |cut -d' ' -f2 |tr "\\r\\n" " " |sed -e "s/[ ]\+/ /g;s/ $//;s/^/   /" >> $resdir/${outname}.ill
			echo "" >> $resdir/${outname}.ill
			#echo "> Z-axis surface normal.. ${ptsname} | x: $ave_x/$max_x | y: $ave_y/$max_y | z: $ave_z/$max_z"
		fi
		cat $tmpdir/$tmp_ill |perl -anF'#|\n' -e'$n=@F-1if!$n;for(0..$n){push@{$$m[$_]},$F[$_]}END{print map{join"#",@$_,"\n"}@$m}' |tr "#" " " |paste -d " " $timestep - |sed -e "s/ .$//g" >> $resdir/${outname}.ill
	done
done

if [ $views ];then
	for vw in $vwlist; do
		viewname=$(basename $vw .${vw##*.})
		#ulimit -n 3000
		vws="-x $dim -y $dim -vf ${vwdir}/${vw}"
		echo "[$(date +%Y-%m-%d-%H%M%S)]  Generating annual HDR for \"${modelname}--${viewname}--${weaname}\" with an \"MF:4\" setting." >> $log
		# Option -c repeats sampling of each pixel the set amount of times. Should be used with the -pj jitter option.
		vwrays -ff $vws |rcontrib @${opt_view} -ffc -fo $(vwrays -d $vws) -e MF:${granularity} -f reinhart.cal -b rbin -bn Nrbins -o ${resviewdir}/${modelname}--${viewname}_%04d.hdr -m sky_glow -w ${modelname}_dmx.oct
		for wea in $wealist; do
			weaname=$(basename $wea .${wea##*.})
			echo "[$(date +%Y-%m-%d-%H%M%S)]  Running dctimestep on: ${resviewdir}/${modelname}--${inputname}--${weaname_s}_vmx/${div}_%04d.hdr $wea" >> $log
			dctimestep -o ${resdir}/${modelname}--${viewname}-${weaname}_%04d.hdr ${resviewdir}/${modelname}--${viewname}_%04d.hdr $weadir/${weaname}--m${granularity}${rztag}.smx
			echo "[$(date +%Y-%m-%d-%H%M%S)]  dctimestep finished: ${modelname}--${inputname}--${weaname}" >> $log
		done
	done
fi
timeend=$(date +%Y-%m-%d-%H%M%S)
echo "[$(date +%Y-%m-%d-%H%M%S)]  Finished." >> $log
echo ">  Finished. The results have been written to the ${resdir}/ directory and the log to the ${logdir}/ directory."

#EOF


